# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from azure.cli.testsdk import (ScenarioTest, JMESPathCheck, ResourceGroupPreparer,
                               StorageAccountPreparer, api_version_constraint, live_only)
from azure.cli.core.profiles import ResourceType
from ..storage_test_util import StorageScenarioMixin
from knack.util import CLIError


@api_version_constraint(ResourceType.MGMT_STORAGE, min_api='2016-12-01')
class StorageAccountTests(StorageScenarioMixin, ScenarioTest):
    @api_version_constraint(ResourceType.MGMT_STORAGE, min_api='2017-06-01')
    @ResourceGroupPreparer(name_prefix='cli_test_storage_service_endpoints')
    @StorageAccountPreparer()
    def test_storage_account_service_endpoints(self, resource_group, storage_account):
        kwargs = {
            'rg': resource_group,
            'acc': storage_account,
            'vnet': 'vnet1',
            'subnet': 'subnet1'
        }
        self.cmd('storage account create -g {rg} -n {acc} --bypass Metrics --default-action Deny'.format(**kwargs),
                 checks=[
                     JMESPathCheck('networkRuleSet.bypass', 'Metrics'),
                     JMESPathCheck('networkRuleSet.defaultAction', 'Deny')])
        self.cmd('storage account update -g {rg} -n {acc} --bypass Logging --default-action Allow'.format(**kwargs),
                 checks=[
                     JMESPathCheck('networkRuleSet.bypass', 'Logging'),
                     JMESPathCheck('networkRuleSet.defaultAction', 'Allow')])
        self.cmd('storage account update -g {rg} -n {acc} --set networkRuleSet.default_action=deny'.format(**kwargs),
                 checks=[
                     JMESPathCheck('networkRuleSet.bypass', 'Logging'),
                     JMESPathCheck('networkRuleSet.defaultAction', 'Deny')])

        self.cmd('network vnet create -g {rg} -n {vnet} --subnet-name {subnet}'.format(**kwargs))
        self.cmd(
            'network vnet subnet update -g {rg} --vnet-name {vnet} -n {subnet} --service-endpoints Microsoft.Storage'.format(
                **kwargs))

        self.cmd('storage account network-rule add -g {rg} --account-name {acc} --ip-address 25.1.2.3'.format(**kwargs))
        self.cmd(
            'storage account network-rule add -g {rg} --account-name {acc} --ip-address 25.2.0.0/24'.format(**kwargs))
        self.cmd(
            'storage account network-rule add -g {rg} --account-name {acc} --vnet-name {vnet} --subnet {subnet}'.format(
                **kwargs))
        self.cmd('storage account network-rule list -g {rg} --account-name {acc}'.format(**kwargs), checks=[
            JMESPathCheck('length(ipRules)', 2),
            JMESPathCheck('length(virtualNetworkRules)', 1)
        ])
        self.cmd(
            'storage account network-rule remove -g {rg} --account-name {acc} --ip-address 25.1.2.3'.format(**kwargs))
        self.cmd(
            'storage account network-rule remove -g {rg} --account-name {acc} --vnet-name {vnet} --subnet {subnet}'.format(
                **kwargs))
        self.cmd('storage account network-rule list -g {rg} --account-name {acc}'.format(**kwargs), checks=[
            JMESPathCheck('length(ipRules)', 1),
            JMESPathCheck('length(virtualNetworkRules)', 0)
        ])

    @api_version_constraint(ResourceType.MGMT_STORAGE, min_api='2017-06-01')
    @ResourceGroupPreparer(location='southcentralus')
    def test_create_storage_account_with_assigned_identity(self, resource_group):
        name = self.create_random_name(prefix='cli', length=24)
        cmd = 'az storage account create -n {} -g {} --sku Standard_LRS --assign-identity'.format(name, resource_group)
        result = self.cmd(cmd).get_output_in_json()

        self.assertIn('identity', result)
        self.assertTrue(result['identity']['principalId'])
        self.assertTrue(result['identity']['tenantId'])

    @api_version_constraint(ResourceType.MGMT_STORAGE, min_api='2017-06-01')
    @ResourceGroupPreparer(location='southcentralus')
    def test_update_storage_account_with_assigned_identity(self, resource_group):
        name = self.create_random_name(prefix='cli', length=24)
        create_cmd = 'az storage account create -n {} -g {} --sku Standard_LRS'.format(name, resource_group)
        self.cmd(create_cmd, checks=[JMESPathCheck('identity', None)])

        update_cmd = 'az storage account update -n {} -g {} --assign-identity'.format(name, resource_group)
        result = self.cmd(update_cmd).get_output_in_json()

        self.assertIn('identity', result)
        self.assertTrue(result['identity']['principalId'])
        self.assertTrue(result['identity']['tenantId'])

    @ResourceGroupPreparer(parameter_name_for_location='location')
    def test_create_storage_account(self, resource_group, location):
        name = self.create_random_name(prefix='cli', length=24)

        self.cmd('az storage account create -n {} -g {} --sku {} -l {}'.format(
            name, resource_group, 'Standard_LRS', location))

        self.cmd('storage account check-name --name {}'.format(name), checks=[
            JMESPathCheck('nameAvailable', False),
            JMESPathCheck('reason', 'AlreadyExists')
        ])

        self.cmd('storage account list -g {}'.format(resource_group), checks=[
            JMESPathCheck('[0].location', 'westus'),
            JMESPathCheck('[0].sku.name', 'Standard_LRS'),
            JMESPathCheck('[0].resourceGroup', resource_group)
        ])

        self.cmd('az storage account show -n {} -g {}'.format(name, resource_group), checks=[
            JMESPathCheck('name', name),
            JMESPathCheck('location', location),
            JMESPathCheck('sku.name', 'Standard_LRS'),
            JMESPathCheck('kind', 'Storage')
        ])

        self.cmd('az storage account show -n {}'.format(name), checks=[
            JMESPathCheck('name', name),
            JMESPathCheck('location', location),
            JMESPathCheck('sku.name', 'Standard_LRS'),
            JMESPathCheck('kind', 'Storage')
        ])

        self.cmd('storage account show-connection-string -g {} -n {} --protocol http'.format(
            resource_group, name), checks=[
                JMESPathCheck("contains(connectionString, 'https')", False),
                JMESPathCheck("contains(connectionString, '{}')".format(name), True)])

        self.cmd('storage account update -g {} -n {} --tags foo=bar cat'
                 .format(resource_group, name),
                 checks=JMESPathCheck('tags', {'cat': '', 'foo': 'bar'}))
        self.cmd('storage account update -g {} -n {} --sku Standard_GRS --tags'
                 .format(resource_group, name),
                 checks=[JMESPathCheck('tags', {}),
                         JMESPathCheck('sku.name', 'Standard_GRS')])
        self.cmd('storage account update -g {} -n {} --set tags.test=success'
                 .format(resource_group, name),
                 checks=JMESPathCheck('tags', {'test': 'success'}))
        self.cmd('storage account delete -g {} -n {} --yes'.format(resource_group, name))
        self.cmd('storage account check-name --name {}'.format(name),
                 checks=JMESPathCheck('nameAvailable', True))

    @api_version_constraint(ResourceType.MGMT_STORAGE, min_api='2017-10-01')
    @ResourceGroupPreparer(parameter_name_for_location='location', location='southcentralus')
    def test_create_storage_account_v2(self, resource_group, location):
        self.kwargs.update({
            'name': self.create_random_name(prefix='cli', length=24),
            'loc': location
        })

        self.cmd('storage account create -n {name} -g {rg} -l {loc} --kind StorageV2',
                 checks=[JMESPathCheck('kind', 'StorageV2')])

        self.cmd('storage account check-name --name {name}', checks=[
            JMESPathCheck('nameAvailable', False),
            JMESPathCheck('reason', 'AlreadyExists')
        ])

    @api_version_constraint(ResourceType.MGMT_STORAGE, min_api='2016-01-01')
    @ResourceGroupPreparer(location='southcentralus')
    def test_storage_create_default_sku(self, resource_group):
        name = self.create_random_name(prefix='cli', length=24)
        create_cmd = 'az storage account create -n {} -g {}'.format(name, resource_group)
        self.cmd(create_cmd, checks=[JMESPathCheck('sku.name', 'Standard_RAGRS')])

    def test_show_usage(self):
        self.cmd('storage account show-usage -l westus', checks=JMESPathCheck('name.value', 'StorageAccounts'))

    def test_show_usage_no_location(self):
        with self.assertRaises(SystemExit):
            self.cmd('storage account show-usage')

    @ResourceGroupPreparer()
    @StorageAccountPreparer()
    def test_logging_operations(self, resource_group, storage_account):
        connection_string = self.cmd(
            'storage account show-connection-string -g {} -n {} -otsv'.format(resource_group, storage_account)).output

        self.cmd('storage logging show --connection-string {}'.format(connection_string), checks=[
            JMESPathCheck('blob.read', False),
            JMESPathCheck('blob.retentionPolicy.enabled', False)
        ])

        self.cmd('storage logging update --services b --log r --retention 1 '
                 '--service b --connection-string {}'.format(connection_string))

        self.cmd('storage logging show --connection-string {}'.format(connection_string), checks=[
            JMESPathCheck('blob.read', True),
            JMESPathCheck('blob.retentionPolicy.enabled', True),
            JMESPathCheck('blob.retentionPolicy.days', 1)
        ])

    @ResourceGroupPreparer()
    @StorageAccountPreparer()
    def test_metrics_operations(self, resource_group, storage_account_info):
        self.storage_cmd('storage metrics show', storage_account_info) \
            .assert_with_checks(JMESPathCheck('file.hour.enabled', True),
                                JMESPathCheck('file.minute.enabled', False))

        self.storage_cmd('storage metrics update --services f --api true --hour true --minute true --retention 1 ',
                         storage_account_info)

        self.storage_cmd('storage metrics show', storage_account_info).assert_with_checks(
            JMESPathCheck('file.hour.enabled', True),
            JMESPathCheck('file.minute.enabled', True))

    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='account_1')
    @StorageAccountPreparer(parameter_name='account_2')
    def test_list_storage_accounts(self, account_1, account_2):
        accounts_list = self.cmd('az storage account list').get_output_in_json()
        assert len(accounts_list) >= 2
        assert next(acc for acc in accounts_list if acc['name'] == account_1)
        assert next(acc for acc in accounts_list if acc['name'] == account_2)

    @ResourceGroupPreparer()
    @StorageAccountPreparer()
    def test_renew_account_key(self, resource_group, storage_account):
        original_keys = self.cmd('storage account keys list -g {} -n {}'
                                 .format(resource_group, storage_account)).get_output_in_json()
        # key1 = keys_result[0]
        # key2 = keys_result[1]
        assert original_keys[0] and original_keys[1]

        renewed_keys = self.cmd('storage account keys renew -g {} -n {} --key primary'
                                .format(resource_group, storage_account)).get_output_in_json()
        print(renewed_keys)
        print(original_keys)
        assert renewed_keys[0] != original_keys[0]
        assert renewed_keys[1] == original_keys[1]

        original_keys = renewed_keys
        renewed_keys = self.cmd('storage account keys renew -g {} -n {} --key secondary'
                                .format(resource_group, storage_account)).get_output_in_json()
        assert renewed_keys[0] == original_keys[0]
        assert renewed_keys[1] != original_keys[1]

    @ResourceGroupPreparer()
    @StorageAccountPreparer()
    def test_create_account_sas(self, storage_account):
        sas = self.cmd('storage account generate-sas --resource-types o --services b '
                       '--expiry 2046-12-31T08:23Z --permissions r --https-only --account-name {}'
                       .format(storage_account)).output
        self.assertIn('sig=', sas, 'SAS token {} does not contain sig segment'.format(sas))
        self.assertIn('se=', sas, 'SAS token {} does not contain se segment'.format(sas))

    def test_list_locations(self):
        self.cmd('az account list-locations',
                 checks=[JMESPathCheck("[?name=='westus'].displayName | [0]", 'West US')])

    @ResourceGroupPreparer(location='southcentralus')
    @StorageAccountPreparer(location='southcentralus')
    def test_customer_managed_key(self, resource_group, storage_account):
        self.kwargs = {'rg': resource_group, 'sa': storage_account, 'vt': self.create_random_name('clitest', 24)}

        self.kwargs['vid'] = self.cmd('az keyvault create -n {vt} -g {rg} '
                                      '-otsv --query id').output.rstrip('\n')
        self.kwargs['vtn'] = self.cmd('az keyvault show -n {vt} -g {rg} '
                                      '-otsv --query properties.vaultUri').output.strip('\n')
        self.kwargs['ver'] = self.cmd("az keyvault key create -n testkey -p software --vault-name {vt} "
                                      "-otsv --query 'key.kid'").output.rsplit('/', 1)[1].rstrip('\n')
        self.kwargs['oid'] = self.cmd("az storage account update -n {sa} -g {rg} --assign-identity "
                                      "-otsv --query 'identity.principalId'").output.strip('\n')

        self.cmd('az keyvault set-policy -n {vt} --object-id {oid} -g {rg} '
                 '--key-permissions get wrapKey unwrapKey recover')
        self.cmd('az keyvault update -n {vt} -g {rg} --set properties.enableSoftDelete=true')
        self.cmd('az resource update --id {vid} --set properties.enablePurgeProtection=true')
        self.cmd('az storage account update -n {sa} -g {rg} '
                 '--encryption-key-source Microsoft.Keyvault '
                 '--encryption-key-vault {vtn} '
                 '--encryption-key-name testkey '
                 '--encryption-key-version {ver} ')

    @ResourceGroupPreparer()
    @StorageAccountPreparer()
    def test_storage_account_show_exit_codes(self, resource_group, storage_account):
        self.kwargs = {'rg': resource_group, 'sa': storage_account}

        self.assertEqual(self.cmd('storage account show -g {rg} -n {sa}').exit_code, 0)

        with self.assertRaises(SystemExit) as ex:
            self.cmd('storage account show text_causing_parsing_error')
        self.assertEqual(ex.exception.code, 2)

        with self.assertRaises(SystemExit) as ex:
            self.cmd('storage account show -g fake_group -n {sa}')
        self.assertEqual(ex.exception.code, 3)

        with self.assertRaises(SystemExit) as ex:
            self.cmd('storage account show -g {rg} -n fake_account')
        self.assertEqual(ex.exception.code, 3)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import ScenarioTest, record_only


@record_only()
class PolicyInsightsTests(ScenarioTest):

    def test_policy_insights(self):
        top_clause = '--top 2'
        filter_clause = '--filter "isCompliant eq false"'
        apply_clause = '--apply "groupby((policyAssignmentId, resourceId), aggregate($count as numRecords))"'
        select_clause = '--select "policyAssignmentId, resourceId, numRecords"'
        order_by_clause = '--order-by "numRecords desc"'
        from_clause = '--from "2018-04-04T00:00:00"'
        to_clause = '--to "2018-05-22T00:00:00"'
        scopes = [
            '-m "azgovtest4"',
            '',
            '-g "defaultresourcegroup-eus"',
            '--resource "/subscriptions/00000000-0000-0000-0000-000000000000/resourcegroups/eastusnsggroup/providers/microsoft.network/networksecuritygroups/eastusnsg/securityrules/allow-joba"',
            '--resource "omssecuritydevkeyvalut" --namespace "microsoft.keyvault" --resource-type "vaults" -g "omssecurityintresourcegroup"',
            '--resource "default" --namespace "microsoft.network" --resource-type "subnets" --parent "virtualnetworks/mms-wcus-vnet" -g "mms-wcus"',
            '-s "335cefd2-ab16-430f-b364-974a170eb1d5"',
            '-d "25bf1e2a-6004-47ad-9bd1-2a40dd6de016"',
            '-a "96e22f7846e94bb186ae3a01"',
            '-a "bc916e4f3ab54030822a11b3" -g "tipkeyvaultresourcegroup" '
        ]

        for scope in scopes:
            events = self.cmd('az policy event list {} {} {} {} {} {} {} {}'.format(
                scope,
                from_clause,
                to_clause,
                filter_clause,
                apply_clause,
                select_clause,
                order_by_clause,
                top_clause)).get_output_in_json()
            assert len(events) >= 0

            states = self.cmd('az policy state list {} {} {} {} {} {} {} {}'.format(
                scope,
                from_clause,
                to_clause,
                filter_clause,
                apply_clause,
                select_clause,
                order_by_clause,
                top_clause)).get_output_in_json()
            assert len(states) >= 0

            summary = self.cmd('az policy state summarize {} {} {} {} {}'.format(
                scope,
                from_clause,
                to_clause,
                filter_clause,
                top_clause)).get_output_in_json()
            assert summary["results"] is not None
            assert len(summary["policyAssignments"]) >= 0
            if len(summary["policyAssignments"]) > 0:
                assert summary["policyAssignments"][0]["results"] is not None
                assert len(summary["policyAssignments"][0]["policyDefinitions"]) >= 0
                if len(summary["policyAssignments"][0]["policyDefinitions"]) > 0:
                    assert summary["policyAssignments"][0]["policyDefinitions"][0]["results"] is not None



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import mock
import time

from azure.cli.core.util import CLIError
from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer, StorageAccountPreparer


class AmsJobTests(ScenarioTest):
    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_for_create')
    def test_ams_job(self, resource_group, storage_account_for_create):
        amsname = self.create_random_name(prefix='ams', length=12)

        self.kwargs.update({
            'amsname': amsname,
            'storageAccount': storage_account_for_create,
            'location': 'westus2'
        })

        self.cmd('az ams account create -n {amsname} -g {rg} --storage-account {storageAccount} -l {location}')

        assetName = self.create_random_name(prefix='asset', length=12)

        self.kwargs.update({
            'assetName': assetName
        })

        self.cmd('az ams asset create -a {amsname} -n {assetName} -g {rg}')

        transformName = self.create_random_name(prefix='tra', length=10)

        self.kwargs.update({
            'transformName': transformName,
            'presetName': 'AACGoodQualityAudio',
            'label': 'someLabel'
        })

        self.cmd('az ams transform create -a {amsname} -n {transformName} -g {rg} --preset {presetName}')

        jobName = self.create_random_name(prefix='job', length=10)

        self.kwargs.update({
            'jobName': jobName,
            'priority': 'High',
            'correlationData': 'foo=bar baz=fuzz',
            'outputLabel': 'outputLabel'
        })

        self.cmd('az ams job start -t {transformName} -a {amsname} -g {rg} -n {jobName} --input-asset-name {assetName} --output-assets {assetName}={outputLabel} --priority {priority} --label {label} --correlation-data {correlationData}', checks=[
            self.check('name', '{jobName}'),
            self.check('resourceGroup', '{rg}'),
            self.check('input.label', '{label}'),
            self.check('priority', '{priority}'),
            self.check('outputs[0].label', '{outputLabel}')
        ])

        self.cmd('az ams job show -a {amsname} -n {jobName} -g {rg} -t {transformName}', checks=[
            self.check('name', '{jobName}'),
            self.check('resourceGroup', '{rg}'),
            self.check('priority', '{priority}'),
            self.check('length(correlationData)', 2),
            self.check('outputs[0].label', '{outputLabel}')
        ])

        list = self.cmd('az ams job list -a {amsname} -g {rg} -t {transformName}').get_output_in_json()
        assert len(list) > 0

        self.kwargs.update({
            'priority': 'Low',
            'description': 'someDescription'
        })

        self.cmd('az ams job update -a {amsname} -n {jobName} -g {rg} -t {transformName} --description {description} --priority {priority}', checks=[
            self.check('priority', '{priority}'),
            self.check('description', '{description}')
        ])

        self.cmd('az ams job cancel -n {jobName} -a {amsname} -g {rg} -t {transformName}')

        job = self.cmd('az ams job show -a {amsname} -n {jobName} -g {rg} -t {transformName}', checks=[
            self.check('name', '{jobName}'),
            self.check('resourceGroup', '{rg}'),
            self.check('priority', '{priority}')
        ]).get_output_in_json()

        assert job['state'] == 'Canceled' or job['state'] == 'Canceling'

        _RETRY_TIMES = 5
        for l in range(0, _RETRY_TIMES):
            try:
                self.cmd('az ams job delete -n {jobName} -a {amsname} -g {rg} -t {transformName}')
                break
            except Exception:  # pylint: disable=broad-except
                if l < _RETRY_TIMES:
                    time.sleep(5)
                else:
                    raise



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import ScenarioTest, StorageAccountPreparer, ResourceGroupPreparer, record_only


class AcrTaskCommandsTests(ScenarioTest):

    @ResourceGroupPreparer()
    def test_acr_task(self, resource_group):
        self.kwargs.update({
            'registry_name': self.create_random_name('clireg', 20),
            'task_name': 'testTask',
            'rg_loc': 'westcentralus',
            'sku': 'Standard',
            # This token requires 'admin:repo_hook' access. Recycle the token after recording tests.
            'git_access_token': 'c79e207682b7aeea3d94313f66f0dc328c1c4a62',
            'context': 'https://github.com/ankurkhemani/acr-helloworld.git',
            'file': './AcrHelloworld/Dockerfile',
            'image': 'testtask:v1',
            'commit_trigger_status': 'Enabled',
            'git_source_control_type': 'Github'
        })
        self.cmd('acr create -n {registry_name} -g {rg} -l {rg_loc} --sku {sku}',
                 checks=[self.check('name', '{registry_name}'),
                         self.check('location', '{rg_loc}'),
                         self.check('adminUserEnabled', False),
                         self.check('sku.name', 'Standard'),
                         self.check('sku.tier', 'Standard'),
                         self.check('provisioningState', 'Succeeded')])

        # Create a docker build task.
        self.cmd('acr task create -n {task_name} -r {registry_name} --git-access-token {git_access_token} --context {context} --image {image} -f {file}',
                 checks=[self.check('name', '{task_name}'),
                         self.check('location', '{rg_loc}'),
                         self.check('platform.os', 'Linux'),
                         self.check('agentConfiguration.cpu', 2),
                         self.check('provisioningState', 'Succeeded'),
                         self.check('status', 'Enabled'),
                         self.check('timeout', 3600),
                         self.check('trigger.sourceTriggers[0].sourceRepository.repositoryUrl', '{context}'),
                         self.check('trigger.sourceTriggers[0].sourceRepository.sourceControlType', '{git_source_control_type}'),
                         self.check('trigger.sourceTriggers[0].status', '{commit_trigger_status}'),
                         self.check('step.dockerFilePath', '{file}'),
                         self.check('step.imageNames', ['testtask:v1']),
                         self.check('step.arguments', []),
                         self.check('step.isPushEnabled', True),
                         self.check('step.noCache', False)])

        self.cmd('acr task list -r {registry_name}',
                 checks=[self.check('[0].name', '{task_name}')])

        # trigger a run from the task
        response = self.cmd('acr task run -n {task_name} -r {registry_name} --no-logs',
                            checks=[self.check('type', 'Microsoft.ContainerRegistry/registries/runs'),
                                    self.check('status', 'Succeeded')]).get_output_in_json()

        self.kwargs.update({
            'run_id': response['runId']
        })

        # list all runs for the task
        self.cmd('acr task list-runs -n {task_name} -r {registry_name}',
                 checks=[self.check('[0].type', 'Microsoft.ContainerRegistry/registries/runs'),
                         self.check('[0].runId', '{run_id}'),
                         self.check('[0].isArchiveEnabled', False)])

        # show task properties
        self.cmd('acr task show -n {task_name} -r {registry_name}',
                 checks=[self.check('name', '{task_name}')])

        # update the first task using non-default parameter values
        self.cmd('acr task update -n {task_name} -r {registry_name} --cpu 1',
                 checks=[self.check('name', '{task_name}'),
                         self.check('agentConfiguration.cpu', 1)])

        # update a run of the first task
        self.cmd('acr task update-run -r {registry_name} --run-id {run_id} --no-archive false',
                 checks=[self.check('type', 'Microsoft.ContainerRegistry/registries/runs'),
                         self.check('runId', '{run_id}'),
                         self.check('isArchiveEnabled', True),
                         self.check('provisioningState', 'Succeeded')])

        # test task delete
        self.cmd('acr task delete -n {task_name} -r {registry_name}')

        # test acr delete
        self.cmd('acr delete -n {registry_name} -g {rg}')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import collections
import unittest
from contextlib import contextmanager

import os
# noinspection PyProtectedMember
from azure.cli.command_modules.batchai.custom import (
    _patch_mount_volumes,
    _update_nodes_information,
    _update_user_account_settings,
    _add_azure_container_to_mount_volumes,
    _add_azure_file_share_to_mount_volumes,
    _add_nfs_to_mount_volumes,
    _is_on_mount_point,
    _ensure_subnet_is_valid,
    _get_image_reference,
    _list_node_setup_files_for_cluster,
    _generate_auto_storage_account_name,
    _add_setup_task,
    _get_effective_resource_parameters)
from azure.cli.core.util import CLIError
from azure.cli.core.mock import DummyCli
from azure.mgmt.batchai.models import (
    Cluster, ClusterCreateParameters, UserAccountSettings, MountVolumes, NodeSetup, FileServer,
    FileServerCreateParameters, AzureFileShareReference, AzureBlobFileSystemReference, AzureStorageCredentialsInfo,
    SshConfiguration, ImageReference, ScaleSettings, ManualScaleSettings, AutoScaleSettings, ResourceId, SetupTask
)
# Some valid ssh public key value.
from mock import MagicMock, patch

SSH_KEY = 'ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQDKUDnWeK6rx36apNE9ij1iAXn68FKXLTW0009XK/7dyMewlNfXi6Z2opnxHDD' \
          'YWucMluU0dsvBR22OuYH2RHriPJTi1jN3aZ0zZSrlH/W4MSNQKlG/AnrjJPA3xfYjIKLGuKBqSIvMsmrkuDfIfMaDnPcxb+GzM1' \
          '0L5epRhoP5FwqaQbLqp640YzFSLqMChz7E6RG54CpEm+MRvA7lj9nD9AlYnfRQAKj2thsFrkz7AlYLQ1ug8vV+1Y3xSKDbo5vy6' \
          'oqM3QKeLiUiyN9Ff1Rq4uYnrqJqcWN1ADfiPGZZjEStOkRgG3V6JrpIN+z0Zl3n+sjrH+CKwrYmyni6D41L'


@contextmanager
def _given_env_variable(name, value):
    os.environ[name] = value
    try:
        yield
    finally:
        os.environ.pop(name, None)


def _get_mock_storage_accounts_and_keys(accounts_and_keys):
    """Creates a mock storage client which knows about given storage accounts and keys"""
    Endpoints = collections.namedtuple('Endpoints', 'file')
    Account = collections.namedtuple('Account', 'id name primary_endpoints')
    mock_storage_client = MagicMock()
    mock_storage_client.storage_accounts.list = MagicMock(
        return_value=[Account(
            '/subscriptions/000/resourceGroups/rg/providers/Microsoft.Storage/storageAccounts/{0}'.format(a),
            a, Endpoints('https://{0}.file.core.windows.net/'.format(a)))
            for a in accounts_and_keys.keys()])
    Keys = collections.namedtuple('Keys', 'keys')
    Key = collections.namedtuple('Key', 'value')
    mock_storage_client.storage_accounts.list_keys = MagicMock(
        side_effect=lambda _, account: Keys([Key(accounts_and_keys.get(account, None))]))
    return mock_storage_client


def _data_file(filename):
    path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data', filename)
    return path.replace('\\', '\\\\')


class TestBatchAICustom(unittest.TestCase):

    def test_batchai_update_cluster_user_account_settings_using_config(self):
        params = ClusterCreateParameters(
            vm_size=None,
            user_account_settings=UserAccountSettings(
                admin_user_name='name',
                admin_user_password='password'))
        actual = _update_user_account_settings(params, None, None, None)
        self.assertEquals(params, actual)

    def test_batchai_update_cluster_user_account_settings_using_command_line(self):
        params = ClusterCreateParameters(vm_size=None, user_account_settings=None)
        actual = _update_user_account_settings(params, 'name', SSH_KEY, 'password')
        self.assertEquals(actual.user_account_settings,
                          UserAccountSettings(
                              admin_user_name='name',
                              admin_user_ssh_public_key=SSH_KEY,
                              admin_user_password='password'))

    def test_batchai_update_cluster_user_account_settings_using_command_line_overriding_config(self):
        params = ClusterCreateParameters(
            vm_size=None,
            user_account_settings=UserAccountSettings(
                admin_user_name='old_name',
                admin_user_ssh_public_key='old key',
                admin_user_password='old_password'))
        actual = _update_user_account_settings(params, 'name', SSH_KEY, 'password')
        self.assertEquals(actual.user_account_settings,
                          UserAccountSettings(
                              admin_user_name='name',
                              admin_user_ssh_public_key=SSH_KEY,
                              admin_user_password='password'))

    def test_batchai_update_cluster_user_account_settings_using_command_line_when_key_is_path(self):
        params = ClusterCreateParameters(vm_size=None, user_account_settings=None)
        actual = _update_user_account_settings(params, 'user', _data_file('key.txt'), None)
        self.assertEquals(actual.user_account_settings,
                          UserAccountSettings(
                              admin_user_name='user',
                              admin_user_ssh_public_key=SSH_KEY))

    def test_batchai_update_cluster_user_account_settings_using_command_line_when_key_is_wrong(self):
        params = ClusterCreateParameters(vm_size=None, user_account_settings=None)
        with self.assertRaisesRegexp(CLIError, 'Incorrect ssh public key value'):
            _update_user_account_settings(params, 'user', 'my key', None)

    @patch('getpass.getuser')
    def test_batchai_update_cluster_user_account_settings_using_current_user(self, getuser):
        getuser.return_value = 'current_user'
        params = ClusterCreateParameters(vm_size=None, user_account_settings=None)
        actual = _update_user_account_settings(params, None, None, 'password')
        self.assertEquals(actual.user_account_settings,
                          UserAccountSettings(
                              admin_user_name='current_user',
                              admin_user_password='password'))

    @patch('getpass.getuser')
    @patch('azure.cli.command_modules.batchai.custom._get_default_ssh_public_key_location')
    def test_batchai_update_cluster_user_account_settings_using_current_user_and_default_ssh_key(
            self, get_default_ssh_public_key_location, getuser):
        get_default_ssh_public_key_location.return_value = _data_file('key.txt')
        getuser.return_value = 'current_user'
        params = ClusterCreateParameters(vm_size=None, user_account_settings=None)
        actual = _update_user_account_settings(params, None, None, None)
        self.assertEquals(actual.user_account_settings,
                          UserAccountSettings(
                              admin_user_name='current_user',
                              admin_user_ssh_public_key=SSH_KEY))

    def test_batchai_update_file_server_user_account_settings_using_config(self):
        params = FileServerCreateParameters(
            vm_size=None,
            data_disks=None,
            ssh_configuration=SshConfiguration(
                user_account_settings=UserAccountSettings(
                    admin_user_name='name', admin_user_password='password')))
        actual = _update_user_account_settings(params, None, None, None)
        self.assertEquals(params, actual)

    def test_batchai_update_file_server_user_account_settings_using_command_line(self):
        params = FileServerCreateParameters(vm_size=None, data_disks=None, ssh_configuration=None)
        actual = _update_user_account_settings(params, 'name', SSH_KEY, 'password')
        self.assertEquals(actual.ssh_configuration.user_account_settings,
                          UserAccountSettings(
                              admin_user_name='name',
                              admin_user_ssh_public_key=SSH_KEY,
                              admin_user_password='password'))

    def test_batchai_update_file_server_user_account_settings_using_command_line_overriding_config(self):
        params = FileServerCreateParameters(
            vm_size=None,
            data_disks=None,
            ssh_configuration=SshConfiguration(
                user_account_settings=UserAccountSettings(
                    admin_user_name='old_name',
                    admin_user_ssh_public_key='old_key',
                    admin_user_password='old_password')))
        actual = _update_user_account_settings(params, 'name', SSH_KEY, 'password')
        self.assertEquals(actual.ssh_configuration.user_account_settings,
                          UserAccountSettings(
                              admin_user_name='name',
                              admin_user_ssh_public_key=SSH_KEY,
                              admin_user_password='password'))

    def test_batchai_update_file_server_user_account_settings_using_command_line_when_key_is_path(self):
        params = FileServerCreateParameters(vm_size=None, data_disks=None, ssh_configuration=None)
        actual = _update_user_account_settings(params, 'user', _data_file('key.txt'), None)
        self.assertEquals(actual.ssh_configuration.user_account_settings,
                          UserAccountSettings(
                              admin_user_name='user',
                              admin_user_ssh_public_key=SSH_KEY))

    def test_batchai_update_fileserver_user_account_settings_using_command_line_when_key_is_wrong(self):
        params = FileServerCreateParameters(vm_size=None, data_disks=None, ssh_configuration=None)
        with self.assertRaisesRegexp(CLIError, 'Incorrect ssh public key value'):
            _update_user_account_settings(params, 'user', 'my key', None)

    @patch('getpass.getuser')
    def test_batchai_update_file_server_user_account_settings_using_current_user(self, getuser):
        getuser.return_value = 'current_user'
        params = FileServerCreateParameters(vm_size=None, data_disks=None, ssh_configuration=None)
        actual = _update_user_account_settings(params, None, None, 'password')
        self.assertEquals(actual.ssh_configuration.user_account_settings,
                          UserAccountSettings(
                              admin_user_name='current_user',
                              admin_user_password='password'))

    @patch('getpass.getuser')
    @patch('azure.cli.command_modules.batchai.custom._get_default_ssh_public_key_location')
    def test_batchai_update_file_server_user_account_settings_using_current_user_and_default_ssh_key(
            self, get_default_ssh_public_key_location, getuser):
        get_default_ssh_public_key_location.return_value = _data_file('key.txt')
        getuser.return_value = 'current_user'
        params = FileServerCreateParameters(vm_size=None, data_disks=None, ssh_configuration=None)
        actual = _update_user_account_settings(params, None, None, None)
        self.assertEquals(actual.ssh_configuration.user_account_settings,
                          UserAccountSettings(
                              admin_user_name='current_user',
                              admin_user_ssh_public_key=SSH_KEY))

    def test_batchai_patch_mount_volumes_no_volumes(self):
        self.assertIsNone(_patch_mount_volumes(DummyCli(), None))

    def test_batchai_patch_mount_volumes_empty_volumes(self):
        self.assertEqual(MountVolumes(), _patch_mount_volumes(DummyCli(), MountVolumes()))

    def test_batchai_patch_mount_volumes_with_templates_via_command_line_args(self):
        # noinspection PyTypeChecker
        mount_volumes = MountVolumes(
            azure_file_shares=[
                AzureFileShareReference(
                    relative_mount_path='azfiles',
                    account_name='<AZURE_BATCHAI_STORAGE_ACCOUNT>',
                    azure_file_url='https://<AZURE_BATCHAI_STORAGE_ACCOUNT>.file.core.windows.net/share',
                    credentials=AzureStorageCredentialsInfo(account_key='<AZURE_BATCHAI_STORAGE_KEY>')
                ),
                AzureFileShareReference(
                    relative_mount_path='azfiles2',
                    account_name=None,
                    azure_file_url='https://<AZURE_BATCHAI_STORAGE_ACCOUNT>.file.core.windows.net/share2',
                    credentials=None
                )
            ],
            azure_blob_file_systems=[
                AzureBlobFileSystemReference(
                    relative_mount_path='blobfs',
                    container_name='container',
                    account_name='<AZURE_BATCHAI_STORAGE_ACCOUNT>',
                    credentials=AzureStorageCredentialsInfo(account_key='<AZURE_BATCHAI_STORAGE_KEY>')
                ),
                AzureBlobFileSystemReference(
                    relative_mount_path='blobfs2',
                    container_name='container2',
                    account_name='<AZURE_BATCHAI_STORAGE_ACCOUNT>',
                    credentials=None
                ),
            ]
        )
        actual = _patch_mount_volumes(DummyCli(), mount_volumes, 'account', 'key')
        expected = MountVolumes(
            azure_file_shares=[
                AzureFileShareReference(
                    relative_mount_path='azfiles',
                    account_name='account',
                    azure_file_url='https://account.file.core.windows.net/share',
                    credentials=AzureStorageCredentialsInfo(account_key='key')
                ),
                AzureFileShareReference(
                    relative_mount_path='azfiles2',
                    account_name='account',
                    azure_file_url='https://account.file.core.windows.net/share2',
                    credentials=AzureStorageCredentialsInfo(account_key='key')
                ),
            ],
            azure_blob_file_systems=[
                AzureBlobFileSystemReference(
                    relative_mount_path='blobfs',
                    container_name='container',
                    account_name='account',
                    credentials=AzureStorageCredentialsInfo(account_key='key')
                ),
                AzureBlobFileSystemReference(
                    relative_mount_path='blobfs2',
                    container_name='container2',
                    account_name='account',
                    credentials=AzureStorageCredentialsInfo(account_key='key')
                )
            ]
        )
        self.assertEquals(expected, actual)

    def test_batchai_patch_mount_volumes_with_templates_via_env_variables(self):
        # noinspection PyTypeChecker
        mount_volumes = MountVolumes(
            azure_file_shares=[
                AzureFileShareReference(
                    relative_mount_path='azfiles',
                    account_name='<AZURE_BATCHAI_STORAGE_ACCOUNT>',
                    azure_file_url='https://<AZURE_BATCHAI_STORAGE_ACCOUNT>.file.core.windows.net/share',
                    credentials=AzureStorageCredentialsInfo(account_key='<AZURE_BATCHAI_STORAGE_KEY>')
                ),
                AzureFileShareReference(
                    relative_mount_path='azfiles2',
                    account_name=None,
                    azure_file_url='https://<AZURE_BATCHAI_STORAGE_ACCOUNT>.file.core.windows.net/share2',
                    credentials=None
                )
            ],
            azure_blob_file_systems=[
                AzureBlobFileSystemReference(
                    relative_mount_path='blobfs',
                    container_name='container',
                    account_name='<AZURE_BATCHAI_STORAGE_ACCOUNT>',
                    credentials=AzureStorageCredentialsInfo(account_key='<AZURE_BATCHAI_STORAGE_KEY>')
                ),
                AzureBlobFileSystemReference(
                    relative_mount_path='blobfs2',
                    container_name='container2',
                    account_name='<AZURE_BATCHAI_STORAGE_ACCOUNT>',
                    credentials=None
                ),
            ]
        )
        with _given_env_variable('AZURE_BATCHAI_STORAGE_ACCOUNT', 'account'):
            with _given_env_variable('AZURE_BATCHAI_STORAGE_KEY', 'key'):
                actual = _patch_mount_volumes(DummyCli(), mount_volumes)
        expected = MountVolumes(
            azure_file_shares=[
                AzureFileShareReference(
                    relative_mount_path='azfiles',
                    account_name='account',
                    azure_file_url='https://account.file.core.windows.net/share',
                    credentials=AzureStorageCredentialsInfo(account_key='key')
                ),
                AzureFileShareReference(
                    relative_mount_path='azfiles2',
                    account_name='account',
                    azure_file_url='https://account.file.core.windows.net/share2',
                    credentials=AzureStorageCredentialsInfo(account_key='key')
                ),
            ],
            azure_blob_file_systems=[
                AzureBlobFileSystemReference(
                    relative_mount_path='blobfs',
                    container_name='container',
                    account_name='account',
                    credentials=AzureStorageCredentialsInfo(account_key='key')
                ),
                AzureBlobFileSystemReference(
                    relative_mount_path='blobfs2',
                    container_name='container2',
                    account_name='account',
                    credentials=AzureStorageCredentialsInfo(account_key='key')
                )
            ]
        )
        self.assertEquals(expected, actual)

    @patch('azure.cli.command_modules.batchai.custom._get_storage_management_client')
    def test_batchai_patch_mount_volumes_with_credentials(self, get_storage_client):
        # noinspection PyTypeChecker
        mount_volumes = MountVolumes(
            azure_file_shares=[
                AzureFileShareReference(
                    relative_mount_path='azfiles',
                    account_name=None,
                    azure_file_url='https://account1.file.core.windows.net/share',
                    credentials=None
                )
            ],
            azure_blob_file_systems=[
                AzureBlobFileSystemReference(
                    relative_mount_path='blobfs',
                    container_name='container',
                    account_name='account2',
                    credentials=None
                ),
            ]
        )
        get_storage_client.return_value = \
            _get_mock_storage_accounts_and_keys({'account1': 'key1', 'account2': 'key2'})
        actual = _patch_mount_volumes(DummyCli(), mount_volumes)
        expected = MountVolumes(
            azure_file_shares=[
                AzureFileShareReference(
                    relative_mount_path='azfiles',
                    account_name='account1',
                    azure_file_url='https://account1.file.core.windows.net/share',
                    credentials=AzureStorageCredentialsInfo(account_key='key1')
                )
            ],
            azure_blob_file_systems=[
                AzureBlobFileSystemReference(
                    relative_mount_path='blobfs',
                    container_name='container',
                    account_name='account2',
                    credentials=AzureStorageCredentialsInfo(account_key='key2')
                )
            ]
        )
        self.assertEquals(expected, actual)

    @patch('azure.cli.command_modules.batchai.custom._get_storage_management_client')
    def test_batchai_patch_mount_volumes_with_credentials_no_account_found(self, get_storage_client):
        get_storage_client.return_value = _get_mock_storage_accounts_and_keys({})
        # noinspection PyTypeChecker
        mount_volumes = MountVolumes(
            azure_file_shares=[
                AzureFileShareReference(
                    relative_mount_path='azfiles',
                    account_name=None,
                    azure_file_url='https://account1.file.core.windows.net/share',
                    credentials=None
                )
            ]
        )
        with self.assertRaisesRegexp(CLIError, 'Cannot find "account1" storage account'):
            _patch_mount_volumes(DummyCli(), mount_volumes)

    @patch('azure.cli.command_modules.batchai.custom._get_storage_management_client')
    def test_batchai_patch_mount_volumes_with_credentials_no_account_given(
            self, get_storage_client):
        get_storage_client.return_value = _get_mock_storage_accounts_and_keys(
            {})
        # noinspection PyTypeChecker
        mount_volumes = MountVolumes(
            azure_file_shares=[
                AzureFileShareReference(
                    relative_mount_path='azfiles',
                    account_name=None,
                    azure_file_url='https://<AZURE_BATCHAI_STORAGE_ACCOUNT>.file.core.windows.net/share',
                    credentials=None
                )
            ]
        )
        with self.assertRaisesRegexp(CLIError, 'Please configure Azure Storage account name'):
            _patch_mount_volumes(DummyCli(), mount_volumes)

    @patch('azure.cli.command_modules.batchai.custom._get_storage_management_client')
    def test_batchai_patch_mount_volumes_when_no_patching_required(
            self, get_storage_client):
        get_storage_client.return_value = _get_mock_storage_accounts_and_keys(
            {})
        # noinspection PyTypeChecker
        mount_volumes = MountVolumes(
            azure_file_shares=[
                AzureFileShareReference(
                    relative_mount_path='azfiles',
                    account_name='account1',
                    azure_file_url='https://account1.file.core.windows.net/share',
                    credentials=AzureStorageCredentialsInfo(account_key='key')
                )
            ]
        )
        actual = _patch_mount_volumes(DummyCli(), mount_volumes)
        self.assertEquals(mount_volumes, actual)

    def test_batchai_patch_mount_volumes_no_azure_file_url(self):
        # noinspection PyTypeChecker
        mount_volumes = MountVolumes(
            azure_file_shares=[
                AzureFileShareReference(
                    relative_mount_path='azfiles',
                    account_name=None,
                    azure_file_url=None,
                    credentials=None
                )
            ]
        )
        with self.assertRaisesRegexp(CLIError, 'Azure File URL cannot absent or be empty'):
            _patch_mount_volumes(DummyCli(), mount_volumes)

    def test_batchai_patch_mount_volumes_ill_formed_azure_file_url(self):
        # noinspection PyTypeChecker
        mount_volumes = MountVolumes(
            azure_file_shares=[
                AzureFileShareReference(
                    relative_mount_path='azfiles',
                    account_name=None,
                    azure_file_url='http://http://something/is/wrong',
                    credentials=None
                )
            ]
        )
        with self.assertRaisesRegexp(CLIError, 'Ill-formed Azure File URL'):
            _patch_mount_volumes(DummyCli(), mount_volumes)

    def test_batchai_add_nfs_to_mount_volumes_no_relative_mount_path(self):
        with self.assertRaisesRegexp(CLIError, 'File server relative mount path cannot be empty'):
            _add_nfs_to_mount_volumes(MountVolumes(), 'id', '')

    def test_batchai_add_nfs_to_empty_mount_volumes(self):
        mount_volumes = _add_nfs_to_mount_volumes(MountVolumes(), 'id', 'relative_path')
        self.assertEquals('id', mount_volumes.file_servers[0].file_server.id)
        self.assertEquals('relative_path', mount_volumes.file_servers[0].relative_mount_path)
        self.assertEquals('rw', mount_volumes.file_servers[0].mount_options)

    def test_batchai_add_nfs_to_absent_mount_volumes(self):
        mount_volumes = _add_nfs_to_mount_volumes(None, 'id', 'relative_path')
        self.assertEquals('id', mount_volumes.file_servers[0].file_server.id)
        self.assertEquals('relative_path', mount_volumes.file_servers[0].relative_mount_path)
        self.assertEquals('rw', mount_volumes.file_servers[0].mount_options)

    def test_batchai_add_nfs_to_non_empty_mount_volumes(self):
        mount_volumes = _add_nfs_to_mount_volumes(MountVolumes(), 'id0', 'relative_path0')
        mount_volumes = _add_nfs_to_mount_volumes(mount_volumes, 'id1', 'relative_path1')
        for i in range(2):
            self.assertEquals('id{0}'.format(i), mount_volumes.file_servers[i].file_server.id)
            self.assertEquals('relative_path{0}'.format(i), mount_volumes.file_servers[i].relative_mount_path)
            self.assertEquals('rw', mount_volumes.file_servers[i].mount_options)

    def test_batchai_add_azure_file_share_to_mount_volumes_no_account_info(self):
        with self.assertRaisesRegexp(CLIError, 'Please configure Azure Storage account name'):
            _add_azure_file_share_to_mount_volumes(DummyCli(), MountVolumes(), 'share', 'relative_path')

    def test_batchai_add_azure_file_share_to_mount_volumes_no_relative_mount_path(self):
        with self.assertRaisesRegexp(CLIError, 'Azure File share relative mount path cannot be empty'):
            _add_azure_file_share_to_mount_volumes(DummyCli(), MountVolumes(), 'share', '')

    @patch('azure.cli.command_modules.batchai.custom._get_storage_management_client')
    def test_batchai_add_azure_file_share_to_mount_volumes_account_and_key_via_env_variables(
            self, get_storage_client):
        get_storage_client.return_value = _get_mock_storage_accounts_and_keys({'account': 'key'})
        with _given_env_variable('AZURE_BATCHAI_STORAGE_ACCOUNT', 'account'):
            with _given_env_variable('AZURE_BATCHAI_STORAGE_KEY', 'key'):
                actual = _add_azure_file_share_to_mount_volumes(DummyCli(), MountVolumes(), 'share', 'relative_path')
        expected = MountVolumes(azure_file_shares=[
            AzureFileShareReference(
                account_name='account',
                azure_file_url='https://account.file.core.windows.net/share',
                relative_mount_path='relative_path',
                credentials=AzureStorageCredentialsInfo(
                    account_key='key'
                )
            )]
        )
        self.assertEqual(expected, actual)

    @patch('azure.cli.command_modules.batchai.custom._get_storage_management_client')
    def test_batchai_add_azure_file_share_to_mount_volumes_account_and_key_via_command_line_args(
            self, get_storage_client):
        get_storage_client.return_value = \
            _get_mock_storage_accounts_and_keys({'account': 'key'})
        actual = _add_azure_file_share_to_mount_volumes(DummyCli(), MountVolumes(), 'share', 'relative_path',
                                                        'account', 'key')
        expected = MountVolumes(azure_file_shares=[
            AzureFileShareReference(
                account_name='account',
                azure_file_url='https://account.file.core.windows.net/share',
                relative_mount_path='relative_path',
                credentials=AzureStorageCredentialsInfo(
                    account_key='key'
                )
            )]
        )
        self.assertEqual(expected, actual)

    @patch('azure.cli.command_modules.batchai.custom._get_storage_management_client')
    def test_batchai_add_azure_file_share_to_mount_volumes_account_via_command_line_args(self, get_storage_client):
        get_storage_client.return_value = _get_mock_storage_accounts_and_keys({'account': 'key'})
        actual = _add_azure_file_share_to_mount_volumes(DummyCli(), MountVolumes(), 'share', 'relative_path', 'account')
        expected = MountVolumes(azure_file_shares=[
            AzureFileShareReference(
                account_name='account',
                azure_file_url='https://account.file.core.windows.net/share',
                relative_mount_path='relative_path',
                credentials=AzureStorageCredentialsInfo(
                    account_key='key'
                )
            )]
        )
        self.assertEqual(expected, actual)

    @patch('azure.cli.command_modules.batchai.custom._get_storage_management_client')
    def test_batchai_add_azure_file_share_to_absent_mount_volumes(self, get_storage_client):
        get_storage_client.return_value = _get_mock_storage_accounts_and_keys({'account': 'key'})
        actual = _add_azure_file_share_to_mount_volumes(DummyCli(), None, 'share', 'relative_path', 'account')
        expected = MountVolumes(azure_file_shares=[
            AzureFileShareReference(
                account_name='account',
                azure_file_url='https://account.file.core.windows.net/share',
                relative_mount_path='relative_path',
                credentials=AzureStorageCredentialsInfo(
                    account_key='key'
                )
            )]
        )
        self.assertEqual(expected, actual)

    def test_batchai_add_azure_container_mount_volumes_no_account(self):
        with self.assertRaisesRegexp(CLIError, 'Please configure Azure Storage account name'):
            _add_azure_container_to_mount_volumes(DummyCli(), MountVolumes(), 'container', 'relative_path')

    def test_batchai_add_azure_container_mount_volumes_no_relative_path(self):
        with self.assertRaisesRegexp(CLIError, 'Azure Storage Container relative mount path cannot be empty'):
            _add_azure_container_to_mount_volumes(DummyCli(), MountVolumes(), 'container', '')

    def test_batchai_add_azure_container_to_absent_mount_volumes(self):
        with _given_env_variable('AZURE_BATCHAI_STORAGE_ACCOUNT', 'account'):
            with _given_env_variable('AZURE_BATCHAI_STORAGE_KEY', 'key'):
                actual = _add_azure_container_to_mount_volumes(DummyCli(), None, 'container', 'relative_path')
        expected = MountVolumes(azure_blob_file_systems=[
            AzureBlobFileSystemReference(
                account_name='account',
                container_name='container',
                relative_mount_path='relative_path',
                credentials=AzureStorageCredentialsInfo(account_key='key'))])
        self.assertEqual(expected, actual)

    def test_batchai_add_azure_container_mount_volumes_account_and_key_via_env_variables(self):
        with _given_env_variable('AZURE_BATCHAI_STORAGE_ACCOUNT', 'account'):
            with _given_env_variable('AZURE_BATCHAI_STORAGE_KEY', 'key'):
                actual = _add_azure_container_to_mount_volumes(DummyCli(), MountVolumes(), 'container', 'relative_path')
        expected = MountVolumes(azure_blob_file_systems=[
            AzureBlobFileSystemReference(
                account_name='account',
                container_name='container',
                relative_mount_path='relative_path',
                credentials=AzureStorageCredentialsInfo(account_key='key'))])
        self.assertEqual(expected, actual)

    def test_batchai_add_azure_container_mount_volumes_account_and_key_via_cmd_line_args(self):
        actual = _add_azure_container_to_mount_volumes(
            DummyCli(), MountVolumes(), 'container', 'relative_path', 'account', 'key')
        expected = MountVolumes(azure_blob_file_systems=[
            AzureBlobFileSystemReference(
                account_name='account',
                container_name='container',
                relative_mount_path='relative_path',
                credentials=AzureStorageCredentialsInfo(account_key='key'))])
        self.assertEqual(expected, actual)

    @patch('azure.cli.command_modules.batchai.custom._get_storage_management_client')
    def test_batchai_add_azure_container_mount_volumes_account_via_cmd_line_args(self, get_storage_client):
        get_storage_client.return_value = _get_mock_storage_accounts_and_keys({'account': 'key'})
        actual = _add_azure_container_to_mount_volumes(
            DummyCli(), MountVolumes(), 'container', 'relative_path', 'account')
        expected = MountVolumes(azure_blob_file_systems=[
            AzureBlobFileSystemReference(
                account_name='account',
                container_name='container',
                relative_mount_path='relative_path',
                credentials=AzureStorageCredentialsInfo(account_key='key'))])
        self.assertEqual(expected, actual)

    def test_batchai_update_nodes_information(self):
        params = ClusterCreateParameters(location='southcentralus', vm_size='STANDARD_D1',
                                         user_account_settings=UserAccountSettings(admin_user_name='name',
                                                                                   admin_user_password='password'))
        # Update to autoscale Ubuntu DSVM.
        result = _update_nodes_information(params, 'ubuntudsvm', None, 'Standard_NC6', 'dedicated', None, 1, 3)
        self.assertEquals(result.vm_size, 'Standard_NC6')
        self.assertEquals(result.vm_priority, 'dedicated')
        self.assertEquals(
            ImageReference(publisher='microsoft-ads', offer='linux-data-science-vm-ubuntu', sku='linuxdsvmubuntu'),
            result.virtual_machine_configuration.image_reference)
        self.assertEquals(result.scale_settings,
                          ScaleSettings(auto_scale=AutoScaleSettings(minimum_node_count=1, maximum_node_count=3)))

        # Update to manual scale Ubuntu LTS.
        result = _update_nodes_information(params, 'UbuntuLTS', None, 'Standard_NC6', '', None, 2, 2)
        self.assertEquals(result.vm_size, 'Standard_NC6')
        self.assertEquals(result.vm_priority, 'dedicated')
        self.assertEquals(ImageReference(
            publisher='Canonical', offer='UbuntuServer', sku='16.04-LTS'),
            result.virtual_machine_configuration.image_reference
        )
        self.assertEquals(result.scale_settings, ScaleSettings(
            manual=ManualScaleSettings(target_node_count=2)))

        # Update to manual scale Ubuntu LTS.
        result = _update_nodes_information(params, 'UbuntuLTS', None, 'Standard_NC6', '', 2, 2, 2)
        self.assertEquals(result.vm_size, 'Standard_NC6')
        self.assertEquals(result.vm_priority, 'dedicated')
        self.assertEquals(
            ImageReference(publisher='Canonical', offer='UbuntuServer', sku='16.04-LTS'),
            result.virtual_machine_configuration.image_reference)
        self.assertEquals(result.scale_settings, ScaleSettings(manual=ManualScaleSettings(target_node_count=2)))

        # Update to manual scale Ubuntu LTS.
        result = _update_nodes_information(params, 'UbuntuLTS', None, 'Standard_NC6', '', 2, None, None)
        self.assertEquals(result.vm_size, 'Standard_NC6')
        self.assertEquals(result.vm_priority, 'dedicated')
        self.assertEquals(
            ImageReference(publisher='Canonical', offer='UbuntuServer', sku='16.04-LTS'),
            result.virtual_machine_configuration.image_reference
        )
        self.assertEquals(result.scale_settings, ScaleSettings(manual=ManualScaleSettings(target_node_count=2)))

        # Update to auto-scale with initial count Ubuntu LTS.
        result = _update_nodes_information(params, 'UbuntuLTS', None, 'Standard_NC6', '', 1, 0, 2)
        self.assertEquals(result.vm_size, 'Standard_NC6')
        self.assertEquals(result.vm_priority, 'dedicated')
        self.assertEquals(
            ImageReference(publisher='Canonical', offer='UbuntuServer', sku='16.04-LTS'),
            result.virtual_machine_configuration.image_reference)
        self.assertEquals(
            ScaleSettings(
                auto_scale=AutoScaleSettings(minimum_node_count=0, maximum_node_count=2, initial_node_count=1)),
            result.scale_settings)

        # Update image.
        params.scale_settings = ScaleSettings(manual=ManualScaleSettings(target_node_count=2))
        result = _update_nodes_information(params, 'UbuntuDsvm', None, None, '', None, None, None)
        self.assertEquals(
            ImageReference(publisher='microsoft-ads', offer='linux-data-science-vm-ubuntu', sku='linuxdsvmubuntu'),
            result.virtual_machine_configuration.image_reference)
        self.assertEquals(result.scale_settings, ScaleSettings(manual=ManualScaleSettings(target_node_count=2)))

        # Update nothing.
        result = _update_nodes_information(params, None, None, None, '', None, None, None)
        self.assertEqual(params, result)

        # Wrong image.
        with self.assertRaisesRegexp(CLIError, 'Unsupported image alias'):
            _update_nodes_information(params, 'unsupported', None, None, '', None, None, None)

        # No VM size.
        params.vm_size = None
        with self.assertRaisesRegexp(CLIError, 'Please provide VM size'):
            _update_nodes_information(params, 'unsupported', None, None, '', None, None, None)

        # No scale settings.
        params.vm_size = 'Standard_NC6'
        params.scale_settings = None
        with self.assertRaisesRegexp(CLIError, 'Please provide scale setting for the cluster'):
            _update_nodes_information(params, None, None, None, '', None, None, None)

        # Error if only min is specified
        with self.assertRaisesRegexp(CLIError, 'You need to either provide both min and max node'):
            _update_nodes_information(params, 'UbuntuLTS', None, 'Standard_NC6', '', 2, 2, None)

    def test_get_image_or_die_supported_aliases(self):
        self.assertEqual(
            ImageReference(publisher='Canonical', offer='UbuntuServer', sku='16.04-LTS'),
            _get_image_reference('ubuntults', None))
        self.assertEqual(
            ImageReference(publisher='microsoft-ads', offer='linux-data-science-vm-ubuntu', sku='linuxdsvmubuntu'),
            _get_image_reference('ubuntudsvm', None))

    def test_get_image_or_die_unsupported_alias(self):
        with self.assertRaisesRegexp(CLIError, 'Unsupported image alias "ubuntu", supported aliases are'):
            _get_image_reference('ubuntu', None)

    def test_get_image_or_die_with_valid_spec(self):
        self.assertEqual(
            ImageReference(publisher='Canonical', offer='UbuntuServer', sku='16.04-LTS'),
            _get_image_reference('Canonical:UbuntuServer:16.04-LTS:', None))
        self.assertEqual(
            ImageReference(publisher='Canonical', offer='UbuntuServer', sku='16.04-LTS', version='latest'),
            _get_image_reference('Canonical:UbuntuServer:16.04-LTS:latest', None))

    def test_get_image_or_die_with_invalid_spec(self):
        with self.assertRaisesRegexp(CLIError, "--image must have format"):
            _get_image_reference('Canonical:UbuntuServer:16.04-LTS', None)
        with self.assertRaisesRegexp(CLIError, "--image must have format"):
            _get_image_reference('Canonical:UbuntuServer:16.04-LTS:latest:', None)
        with self.assertRaisesRegexp(CLIError, "Image publisher must be provided in --image argument"):
            _get_image_reference(':UbuntuServer:16.04-LTS:latest', None)
        with self.assertRaisesRegexp(CLIError, "Image offer must be provided in --image argument"):
            _get_image_reference('Canonical::16.04-LTS:latest', None)
        with self.assertRaisesRegexp(CLIError, "Image sku must be provided in --image argument"):
            _get_image_reference('Canonical:Offer::latest', None)

    def test_get_image_or_die_with_custom_image_without_version(self):
        # noinspection PyTypeChecker
        self.assertEqual(
            ImageReference(
                publisher='Canonical',
                offer='UbuntuServer',
                sku='16.04-LTS',
                version=None,
                virtual_machine_image_id='/subscriptions/00/resourceGroups/gr/providers/Microsoft.Compute/images/img'
            ),
            _get_image_reference(
                'ubuntults', '/subscriptions/00/resourceGroups/gr/providers/Microsoft.Compute/images/img')
        )

    def test_get_image_or_die_with_custom_image_with_version(self):
        self.assertEqual(
            ImageReference(
                publisher='Canonical',
                offer='UbuntuServer',
                sku='16.04-LTS',
                version='latest',
                virtual_machine_image_id='/subscriptions/00/resourceGroups/gr/providers/Microsoft.Compute/images/img'
            ),
            _get_image_reference(
                'Canonical:UbuntuServer:16.04-LTS:latest',
                '/subscriptions/00/resourceGroups/gr/providers/Microsoft.Compute/images/img'),
        )

    def test_get_image_or_die_with_invalid_custom_image(self):
        with self.assertRaisesRegexp(CLIError, 'Ill-formed custom image resource id'):
            _get_image_reference('ubuntults', 'bla-bla')

    def test_get_image_or_die_with_custom_image_but_without_image(self):
        with self.assertRaisesRegexp(CLIError, 'You need to specify --image argument'):
            _get_image_reference(
                None, '/subscriptions/00/resourceGroups/gr/providers/Microsoft.Compute/images/img')

    def test_validate_subnet_no_subnet(self):
        _ensure_subnet_is_valid(None, None, 'mockrg', 'mockws', 'mocknfs')

    def test_validate_subnet_no_nfs(self):
        _ensure_subnet_is_valid(
            None,
            '/subscriptions/000/resourceGroups/gr/providers/Microsoft.Network/virtualNetworks/vn/subnets/subnet',
            'mockrg', 'mockws', None)

    def test_validate_subnet_wrong_resource_id(self):
        with self.assertRaisesRegexp(CLIError, 'Ill-formed subnet resource id'):
            _ensure_subnet_is_valid(None, 'bla-bla', 'mockrg', 'mockws', 'mocknfs')

    def test_validate_subnet_no_conflicts(self):
        mock_client = MagicMock()
        mock_client.file_servers.get = MagicMock(
            return_value=FileServer(subnet=ResourceId(
                id='/subscriptions/000/resourceGroups/gr/providers/Microsoft.Network/virtualNetworks/vn/subnets/subnet')
            ))
        _ensure_subnet_is_valid(
            mock_client,
            '/subscriptions/000/resourceGroups/gr/providers/Microsoft.Network/virtualNetworks/vn/subnets/subnet',
            'mockrg', 'mockws', 'mocknfs')

    def test_validate_subnet_when_conflict(self):
        mock_client = MagicMock()
        mock_client.file_servers.get = MagicMock(
            return_value=FileServer(
                subnet=ResourceId(id='conflict')
            ))
        with self.assertRaisesRegexp(CLIError, 'Cluster and mounted NFS must be in the same subnet'):
            _ensure_subnet_is_valid(
                mock_client,
                '/subscriptions/000/resourceGroups/gr/providers/Microsoft.Network/virtualNetworks/vn/subnets/subnet',
                'mockrg', 'mockws', 'mocknfs')

    def test_is_on_mount_point_yes(self):
        self.assertTrue(_is_on_mount_point('afs', 'afs'))
        self.assertTrue(_is_on_mount_point('afs/', 'afs'))
        self.assertTrue(_is_on_mount_point('afs', 'afs/'))
        self.assertTrue(_is_on_mount_point('afs/', 'afs/'))
        self.assertTrue(_is_on_mount_point('afs/logs', 'afs'))
        self.assertTrue(_is_on_mount_point('afs/logs/', 'afs'))
        self.assertTrue(_is_on_mount_point('afs/logs', 'afs/'))
        self.assertTrue(_is_on_mount_point('afs/logs/', 'afs'))
        self.assertTrue(_is_on_mount_point('folder/afs/logs', 'folder/afs'))

    def test_is_on_mount_point_no(self):
        self.assertFalse(_is_on_mount_point('somewhere', 'afs'))
        self.assertFalse(_is_on_mount_point('af', 'afs'))
        self.assertFalse(_is_on_mount_point('afs', 'af'))

    def test_list_setup_task_files_for_cluster_when_no_node_setup(self):
        self.assertEqual(_list_node_setup_files_for_cluster(DummyCli(), Cluster(), '.', 60), [])

    def test_list_setup_task_files_for_cluster_when_output_not_on_mounts_root(self):
        with self.assertRaisesRegexp(CLIError, 'List files is supported only for clusters with startup task'):
            cluster = Cluster(
                node_setup=NodeSetup(
                    setup_task=SetupTask(
                        command_line='true',
                        std_out_err_path_prefix='/somewhere')))
            _list_node_setup_files_for_cluster(DummyCli(), cluster, '.', 60)

    def test_list_setup_task_files_for_cluster_when_cluster_doesnt_support_suffix(self):
        with self.assertRaisesRegexp(CLIError, 'List files is not supported for this cluster'):
            cluster = Cluster(
                node_setup=NodeSetup(
                    setup_task=SetupTask(
                        command_line='true',
                        std_out_err_path_prefix='$AZ_BATCHAI_MOUNT_ROOT/nfs')))
            _list_node_setup_files_for_cluster(DummyCli(), cluster, '.', 60)

    def test_list_setup_task_files_for_cluster_when_cluster_has_not_mount_volumes(self):
        with self.assertRaisesRegexp(CLIError, 'List files is supported only for clusters with startup task'):
            cluster = Cluster(
                node_setup=NodeSetup(
                    setup_task=SetupTask(
                        command_line='true',
                        std_out_err_path_prefix='$AZ_BATCHAI_MOUNT_ROOT/nfs')))
            cluster.node_setup.setup_task.std_out_err_path_suffix = 'path/segment'
            _list_node_setup_files_for_cluster(DummyCli(), cluster, '.', 60)

    def test_list_setup_task_files_for_cluster_when_output_not_on_afs_or_bfs(self):
        with self.assertRaisesRegexp(CLIError, 'List files is supported only for clusters with startup task'):
            cluster = Cluster(
                vm_size=None,
                user_account_settings=None,
                node_setup=NodeSetup(
                    setup_task=SetupTask(
                        command_line='true',
                        std_out_err_path_prefix='$AZ_BATCHAI_MOUNT_ROOT/nfs')),
                mount_volumes=MountVolumes(
                    azure_file_shares=[
                        AzureFileShareReference(
                            azure_file_url='https://account.file.core.windows.net/share',
                            relative_mount_path='afs',
                            account_name='some',
                            credentials=None
                        )]
                ))
            cluster.node_setup.setup_task.std_out_err_path_suffix = 'path/segment'
            _list_node_setup_files_for_cluster(DummyCli(), cluster, '.', 60)

    @patch('azure.cli.command_modules.batchai.custom._get_files_from_afs')
    def test_list_setup_task_files_for_cluster_when_output_on_afs(self, get_files_from_afs):
        cluster = Cluster(
            node_setup=NodeSetup(
                setup_task=SetupTask(
                    command_line='true',
                    std_out_err_path_prefix='$AZ_BATCHAI_MOUNT_ROOT/afs')))
        cluster.node_setup.setup_task.std_out_err_path_suffix = 'path/segment'
        cluster.node_setup.mount_volumes = MountVolumes(
            azure_file_shares=[
                AzureFileShareReference(
                    relative_mount_path='afs',
                    account_name='some',
                    azure_file_url='some url',
                    credentials=None)])
        cli_ctx = DummyCli()
        _list_node_setup_files_for_cluster(cli_ctx, cluster, '.', 60)
        get_files_from_afs.assert_called_once_with(
            cli_ctx,
            AzureFileShareReference(
                relative_mount_path='afs',
                account_name='some',
                azure_file_url='some url',
                credentials=None),
            os.path.join('path/segment', '.'), 60)

    @patch('azure.cli.command_modules.batchai.custom._get_files_from_bfs')
    def test_list_setup_task_files_for_cluster_when_output_on_bfs(self, get_files_from_bfs):
        cluster = Cluster(
            node_setup=NodeSetup(
                setup_task=SetupTask(
                    command_line='true',
                    std_out_err_path_prefix='$AZ_BATCHAI_MOUNT_ROOT/bfs')))
        cluster.node_setup.setup_task.std_out_err_path_suffix = 'path/segment'
        cluster.node_setup.mount_volumes = MountVolumes(
            azure_blob_file_systems=[
                AzureBlobFileSystemReference(
                    relative_mount_path='bfs',
                    account_name='some',
                    container_name='container',
                    credentials=None)])
        cli_ctx = DummyCli()
        _list_node_setup_files_for_cluster(cli_ctx, cluster, '.', 60)
        get_files_from_bfs.assert_called_once_with(
            cli_ctx,
            AzureBlobFileSystemReference(
                account_name='some',
                relative_mount_path='bfs',
                container_name='container',
                credentials=None),
            os.path.join('path/segment/', '.'), 60)

    def test_generate_auto_storage_account_name(self):
        names = {_generate_auto_storage_account_name() for i in range(100)}
        # at least 90% of names must be unique
        self.assertTrue(len(names) > 90)

    def test_setup_task_no_output(self):
        with self.assertRaisesRegexp(CLIError, '--setup-task requires providing of --setup-task-output'):
            _add_setup_task('echo hi', None, ClusterCreateParameters(vm_size=None, user_account_settings=None))

    def test_setup_task_added_to_cluster_without_node_setup(self):
        actual = _add_setup_task('echo hi', '/tmp', ClusterCreateParameters(vm_size=None, user_account_settings=None))
        self.assertEqual(actual,
                         ClusterCreateParameters(
                             vm_size=None,
                             user_account_settings=None,
                             node_setup=NodeSetup(
                                 setup_task=SetupTask(
                                     command_line='echo hi',
                                     std_out_err_path_prefix='/tmp',
                                     run_elevated=False
                                 ))))

    def test_setup_task_added_to_cluster_without_setup_task(self):
        actual = _add_setup_task('echo hi', '/tmp', ClusterCreateParameters(
            vm_size=None, user_account_settings=None, node_setup=NodeSetup()))
        self.assertEqual(actual,
                         ClusterCreateParameters(
                             vm_size=None,
                             user_account_settings=None,
                             node_setup=NodeSetup(
                                 setup_task=SetupTask(
                                     command_line='echo hi',
                                     std_out_err_path_prefix='/tmp',
                                     run_elevated=False
                                 ))))

    def test_setup_task_overwrite(self):
        # Should overwrite setup task but keep other parameters of node setup (here we use mount_volumes to check it)
        actual = _add_setup_task('echo hi', '/tmp',
                                 ClusterCreateParameters(
                                     vm_size=None,
                                     user_account_settings=None,
                                     node_setup=NodeSetup(
                                         setup_task=SetupTask(
                                             command_line='different', std_out_err_path_prefix=None),
                                         mount_volumes=[]
                                     )))
        self.assertEqual(actual,
                         ClusterCreateParameters(
                             vm_size=None,
                             user_account_settings=None,
                             node_setup=NodeSetup(
                                 setup_task=SetupTask(
                                     command_line='echo hi',
                                     std_out_err_path_prefix='/tmp',
                                     run_elevated=False
                                 ),
                                 mount_volumes=[])))

    def test_get_effective_resource_parameters_when_resource_id_given(self):
        rg, ws, name = _get_effective_resource_parameters(
            '/subscriptions/xxx/resourceGroups/rg/providers/Microsoft.BatchAI/workspaces/ws/clusters/cl',
            'other_rg', 'other_ws')
        self.assertEqual([rg, ws, name], ['rg', 'ws', 'cl'])

    def test_get_effective_resource_parameters_when_name_given(self):
        rg, ws, name = _get_effective_resource_parameters('cl', 'rg', 'ws')
        self.assertEqual([rg, ws, name], ['rg', 'ws', 'cl'])

    def test_get_effective_resource_parameters_when_none_given(self):
        rg, ws, name = _get_effective_resource_parameters(None, 'rg', 'ws')
        self.assertEqual([rg, ws, name], [None, None, None])



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from azure.cli.testsdk import ResourceGroupPreparer, ScenarioTest, JMESPathCheck
from .scenario_mixin import CdnScenarioMixin


class CdnOriginScenarioTest(CdnScenarioMixin, ScenarioTest):

    @ResourceGroupPreparer()
    def test_origin_list_and_show(self, resource_group):
        profile_name = 'profile123'
        self.profile_create_cmd(resource_group, profile_name)

        endpoint_name = self.create_random_name(prefix='endpoint', length=24)
        origin = 'www.example.com'
        self.endpoint_create_cmd(resource_group, endpoint_name, profile_name, origin)

        checks = [JMESPathCheck('length(@)', 1)]
        origins = self.origin_list_cmd(resource_group, endpoint_name, profile_name, checks=checks)

        origin_name = origins.json_value[0]["name"]
        checks = [JMESPathCheck('name', origin_name)]
        self.origin_show_cmd(resource_group,
                             endpoint_name,
                             profile_name,
                             origin_name,
                             checks=checks)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import os
import tempfile
import unittest
import shutil
import zipfile

import mock

from azure.cli.core.extension import (get_extensions, get_extension_path, extension_exists,
                                      get_extension, get_extension_names, get_extension_modname, ext_compat_with_cli,
                                      ExtensionNotInstalledException, WheelExtension,
                                      EXTENSIONS_MOD_PREFIX, EXT_METADATA_MINCLICOREVERSION, EXT_METADATA_MAXCLICOREVERSION)


# The test extension name
EXT_NAME = 'myfirstcliextension'
EXT_VERSION = '0.0.3+dev'


def _get_test_data_file(filename):
    return os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data', filename)


def _install_test_extension1():  # pylint: disable=no-self-use
    # We extract the extension into place as we aren't testing install here
    zip_file = _get_test_data_file('{}.zip'.format(EXT_NAME))
    zip_ref = zipfile.ZipFile(zip_file, 'r')
    zip_ref.extractall(get_extension_path(EXT_NAME))
    zip_ref.close()


def _install_test_extension2():  # pylint: disable=no-self-use
    # We extract the extension into place as we aren't testing install here
    zip_file = _get_test_data_file('myfirstcliextension_az_extmetadata.zip')
    zip_ref = zipfile.ZipFile(zip_file, 'r')
    zip_ref.extractall(get_extension_path(EXT_NAME))
    zip_ref.close()


class TestExtensionsBase(unittest.TestCase):

    def setUp(self):
        self.ext_dir = tempfile.mkdtemp()
        self.patcher = mock.patch('azure.cli.core.extension.EXTENSIONS_DIR', self.ext_dir)
        self.patcher.start()

    def tearDown(self):
        self.patcher.stop()
        shutil.rmtree(self.ext_dir, ignore_errors=True)


class TestExtensions(TestExtensionsBase):

    def test_no_extensions_dir(self):
        """ Extensions directory doesn't exist """
        shutil.rmtree(self.ext_dir)
        actual = get_extensions()
        self.assertEqual(len(actual), 0)

    def test_no_extensions_in_dir(self):
        """ Directory exists but there are no extensions """
        actual = get_extensions()
        self.assertEqual(len(actual), 0)

    def test_other_files_in_extensions_dir(self):
        tempfile.mkstemp(dir=self.ext_dir)
        actual = get_extensions()
        self.assertEqual(len(actual), 0)

    def test_extension_list(self):
        _install_test_extension1()
        actual = get_extensions()
        self.assertEqual(len(actual), 1)

    def test_extension_exists(self):
        _install_test_extension1()
        actual = extension_exists(EXT_NAME)
        self.assertTrue(actual)

    def test_extension_not_exists(self):
        actual = extension_exists(EXT_NAME)
        self.assertFalse(actual)

    def test_extension_not_exists_2(self):
        _install_test_extension1()
        actual = extension_exists('notanextension')
        self.assertFalse(actual)

    def test_get_extension(self):
        _install_test_extension1()
        actual = get_extension(EXT_NAME)
        self.assertEqual(actual.name, EXT_NAME)

    def test_get_extension_not_installed(self):
        with self.assertRaises(ExtensionNotInstalledException):
            get_extension(EXT_NAME)

    def test_get_extension_names(self):
        _install_test_extension1()
        actual = get_extension_names()
        self.assertEqual(len(actual), 1)
        self.assertEqual(actual[0], EXT_NAME)

    def test_get_extension_modname_no_mods_with_prefix(self):
        tmp_dir = tempfile.mkdtemp()
        with self.assertRaises(AssertionError):
            get_extension_modname(ext_dir=tmp_dir)

    def test_get_extension_modname_okay(self):
        tmp_dir = tempfile.mkdtemp()
        expected_modname = EXTENSIONS_MOD_PREFIX + 'helloworldmod'
        os.makedirs(os.path.join(tmp_dir, expected_modname))
        actual_modname = get_extension_modname(ext_dir=tmp_dir)
        self.assertEqual(expected_modname, actual_modname)

    def test_get_extension_modname_too_many_mods_with_prefix(self):
        tmp_dir = tempfile.mkdtemp()
        modname1 = EXTENSIONS_MOD_PREFIX + 'helloworldmod1'
        modname2 = EXTENSIONS_MOD_PREFIX + 'helloworldmod2'
        os.makedirs(os.path.join(tmp_dir, modname1))
        os.makedirs(os.path.join(tmp_dir, modname2))
        with self.assertRaises(AssertionError):
            get_extension_modname(ext_dir=tmp_dir)

    def test_get_extension_modname_file_with_prefix(self):
        # We should only file a module if it's a directory and not a file even if it has the prefix
        tmp_dir = tempfile.mkdtemp()
        filename = EXTENSIONS_MOD_PREFIX + 'helloworld'
        open(os.path.join(tmp_dir, filename), 'a').close()
        with self.assertRaises(AssertionError):
            get_extension_modname(ext_dir=tmp_dir)

    def test_ext_compat_with_cli_no_v_constraint(self):
        # An extension that does not specify any version constraint on the CLI
        expected_cli_version = '0.0.1'
        azext_metadata = None
        with mock.patch('azure.cli.core.__version__', expected_cli_version):
            is_compatible, cli_version, _, _ = ext_compat_with_cli(azext_metadata)
            self.assertTrue(is_compatible)
            self.assertEqual(cli_version, expected_cli_version)

    def test_ext_compat_with_cli_single_v_constraint(self):
        # An extension that only works with a specific version of the CLI
        expected_cli_version = '0.0.1'
        expected_min_required = '0.0.1'
        expected_max_required = '0.0.1'
        azext_metadata = {EXT_METADATA_MINCLICOREVERSION: expected_min_required,
                          EXT_METADATA_MAXCLICOREVERSION: expected_max_required}
        with mock.patch('azure.cli.core.__version__', expected_cli_version):
            is_compatible, cli_version, min_required, max_required = ext_compat_with_cli(azext_metadata)
            self.assertTrue(is_compatible)
            self.assertEqual(cli_version, expected_cli_version)
            self.assertEqual(min_required, expected_min_required)
            self.assertEqual(max_required, expected_max_required)

    def test_ext_compat_with_cli_only_min_v_constraint(self):
        expected_cli_version = '0.0.5'
        expected_min_required = '0.0.1'
        azext_metadata = {EXT_METADATA_MINCLICOREVERSION: expected_min_required}
        with mock.patch('azure.cli.core.__version__', expected_cli_version):
            is_compatible, _, _, _ = ext_compat_with_cli(azext_metadata)
            self.assertTrue(is_compatible)

    def test_ext_compat_with_cli_failed_bad_min_v_constraint(self):
        expected_cli_version = '0.0.5'
        expected_min_required = '0.0.7'
        azext_metadata = {EXT_METADATA_MINCLICOREVERSION: expected_min_required}
        with mock.patch('azure.cli.core.__version__', expected_cli_version):
            is_compatible, _, _, _ = ext_compat_with_cli(azext_metadata)
            self.assertFalse(is_compatible)

    def test_ext_compat_with_cli_failed_bad_min_but_close_v_constraint(self):
        expected_cli_version = '0.0.5'
        expected_min_required = '0.0.5+dev'
        azext_metadata = {EXT_METADATA_MINCLICOREVERSION: expected_min_required}
        with mock.patch('azure.cli.core.__version__', expected_cli_version):
            is_compatible, _, _, _ = ext_compat_with_cli(azext_metadata)
            self.assertFalse(is_compatible)

    def test_ext_compat_with_cli_only_max_v_constraint(self):
        expected_cli_version = '0.0.5'
        expected_max_required = '0.0.10'
        azext_metadata = {EXT_METADATA_MAXCLICOREVERSION: expected_max_required}
        with mock.patch('azure.cli.core.__version__', expected_cli_version):
            is_compatible, _, _, _ = ext_compat_with_cli(azext_metadata)
            self.assertTrue(is_compatible)

    def test_ext_compat_with_cli_failed_bad_max_v_constraint(self):
        expected_cli_version = '0.0.5'
        expected_max_required = '0.0.3'
        azext_metadata = {EXT_METADATA_MAXCLICOREVERSION: expected_max_required}
        with mock.patch('azure.cli.core.__version__', expected_cli_version):
            is_compatible, _, _, _ = ext_compat_with_cli(azext_metadata)
            self.assertFalse(is_compatible)

    def test_ext_compat_with_cli_failed_bad_max_but_close_v_constraint(self):
        expected_cli_version = '0.0.5'
        expected_max_required = '0.0.5b1'
        azext_metadata = {EXT_METADATA_MAXCLICOREVERSION: expected_max_required}
        with mock.patch('azure.cli.core.__version__', expected_cli_version):
            is_compatible, _, _, _ = ext_compat_with_cli(azext_metadata)
            self.assertFalse(is_compatible)


class TestWheelExtension(TestExtensionsBase):

    def test_wheel_get_all(self):
        _install_test_extension1()
        whl_exts = WheelExtension.get_all()
        self.assertEqual(len(whl_exts), 1)

    def test_wheel_version(self):
        _install_test_extension1()
        ext = get_extension(EXT_NAME)
        self.assertEqual(ext.version, EXT_VERSION)

    def test_wheel_type(self):
        _install_test_extension1()
        ext = get_extension(EXT_NAME)
        self.assertEqual(ext.ext_type, 'whl')

    def test_wheel_metadata1(self):
        _install_test_extension1()
        ext = get_extension(EXT_NAME)
        # There should be no exceptions and metadata should have some value
        self.assertTrue(ext.metadata)

    def test_wheel_metadata2(self):
        _install_test_extension2()
        ext = get_extension(EXT_NAME)
        # There should be no exceptions and metadata should have some value
        self.assertTrue(ext.metadata)
        # We check that we can retrieve any one of the az extension metadata values
        self.assertTrue(ext.metadata.get(EXT_METADATA_MINCLICOREVERSION))


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import unittest
import datetime
import isodate
import mock

from azure.batch import models, operations, BatchServiceClient
from azure.batch.batch_auth import SharedKeyCredentials

from azure.cli.command_modules.batch import _validators
from azure.cli.command_modules.batch import _command_type


class TestObj(object):  # pylint: disable=too-few-public-methods
    pass


class TestBatchValidators(unittest.TestCase):
    # pylint: disable=protected-access

    def __init__(self, methodName):
        super(TestBatchValidators, self).__init__(methodName)

    def test_batch_datetime_format(self):
        obj = _validators.datetime_format("2017-01-24T15:47:24Z")
        self.assertIsInstance(obj, datetime.datetime)

        with self.assertRaises(ValueError):
            _validators.datetime_format("test")

    def test_batch_duration_format(self):
        obj = _validators.duration_format("P3Y6M4DT12H30M5S")
        self.assertIsInstance(obj, isodate.Duration)

        with self.assertRaises(ValueError):
            _validators.duration_format("test")

    def test_batch_metadata_item_format(self):
        meta = _validators.metadata_item_format("name=value")
        self.assertEqual(meta, {'name': 'name', 'value': 'value'})

        with self.assertRaises(ValueError):
            _validators.metadata_item_format("test")

        with self.assertRaises(ValueError):
            _validators.metadata_item_format("name=value=other")

    def test_batch_environment_setting_format(self):
        env = _validators.environment_setting_format("name=value")
        self.assertEqual(env, {'name': 'name', 'value': 'value'})

        with self.assertRaises(ValueError):
            _validators.environment_setting_format("test")

        with self.assertRaises(ValueError):
            _validators.environment_setting_format("name=value=other")

    def test_batch_application_package_reference_format(self):
        ref = _validators.application_package_reference_format("app_1")
        self.assertEqual(ref, {'application_id': 'app_1'})

        ref = _validators.application_package_reference_format("app#1")
        self.assertEqual(ref, {'application_id': 'app', 'version': '1'})

        ref = _validators.application_package_reference_format("app#1#RC")
        self.assertEqual(ref, {'application_id': 'app', 'version': '1#RC'})

    def test_batch_certificate_reference_format(self):
        cert = _validators.certificate_reference_format("thumbprint_lkjsahakjg")
        self.assertEqual(cert, {'thumbprint': 'thumbprint_lkjsahakjg',
                                'thumbprint_algorithm': 'sha1'})

    def test_batch_task_id_ranges_format(self):
        id_range = _validators.task_id_ranges_format("5-10")
        self.assertEqual(id_range, {'start': 5, 'end': 10})

        with self.assertRaises(ValueError):
            _validators.task_id_ranges_format("5")

        with self.assertRaises(ValueError):
            _validators.task_id_ranges_format("test")

        with self.assertRaises(ValueError):
            _validators.environment_setting_format("5-")

        with self.assertRaises(ValueError):
            _validators.environment_setting_format("5-test")

        with self.assertRaises(ValueError):
            _validators.environment_setting_format("start-end")

    def test_batch_resource_file_format(self):
        meta = _validators.resource_file_format("file=source")
        self.assertEqual(meta, {'file_path': 'file', 'blob_source': 'source'})

        meta = _validators.resource_file_format("TestData.zip=https://teststorage.blob.core.windows.net/fgrp-47197bb4/"
                                                "TestData.zip?sv=2015-04-05&sr=b&sig=lk72w%3D&se="
                                                "2017-07-28T21%3A14%3A12Z&sp=rwd")
        self.assertEqual(meta, {
            'file_path': 'TestData.zip',
            'blob_source': ("https://teststorage.blob.core.windows.net/fgrp-47197bb4/"
                            "TestData.zip?sv=2015-04-05&sr=b&sig=lk72w%3D&se=2017-07-28T21%3A14%3A12Z&sp=rwd")})

        with self.assertRaises(ValueError):
            _validators.resource_file_format("file")

    def test_batch_validate_options(self):
        ns = TestObj()
        _validators.validate_options(ns)
        self.assertFalse(hasattr(ns, 'ocp_range'))

        ns.start_range = "100"
        ns.end_range = None
        _validators.validate_options(ns)
        self.assertFalse(hasattr(ns, 'start_range'))
        self.assertFalse(hasattr(ns, 'end_range'))
        self.assertEqual(ns.ocp_range, "bytes=100-")

        del ns.ocp_range
        ns.start_range = None
        ns.end_range = 150
        _validators.validate_options(ns)
        self.assertFalse(hasattr(ns, 'start_range'))
        self.assertFalse(hasattr(ns, 'end_range'))
        self.assertEqual(ns.ocp_range, "bytes=0-150")

        del ns.ocp_range
        ns.start_range = 11
        ns.end_range = 22
        _validators.validate_options(ns)
        self.assertFalse(hasattr(ns, 'start_range'))
        self.assertFalse(hasattr(ns, 'end_range'))
        self.assertEqual(ns.ocp_range, "bytes=11-22")

    def test_batch_validate_file_destination(self):
        ns = TestObj()
        _validators.validate_file_destination(ns)
        self.assertFalse(hasattr(ns, 'destination'))

        ns.destination = os.path.dirname(__file__)
        ns.file_name = "/wd/stdout.txt"
        _validators.validate_file_destination(ns)
        self.assertEqual(ns.destination, os.path.join(os.path.dirname(__file__), 'stdout.txt'))

        ns.destination = __file__
        with self.assertRaises(ValueError):
            _validators.validate_file_destination(ns)

        ns.destination = os.path.join(os.path.dirname(__file__), 'test.txt')
        _validators.validate_file_destination(ns)
        self.assertEqual(ns.destination, os.path.join(os.path.dirname(__file__), 'test.txt'))

        ns.destination = "X:\\test.txt"
        with self.assertRaises(ValueError):
            _validators.validate_file_destination(ns)


class TestBatchParser(unittest.TestCase):
    # pylint: disable=protected-access

    def test_batch_build_prefix(self):
        resolved = _command_type._build_prefix('id', 'id', 'pool')
        self.assertEqual(resolved, 'id')

        resolved = _command_type._build_prefix('id', 'id', 'pool.start_task')
        self.assertEqual(resolved, 'start_task_id')

        resolved = _command_type._build_prefix('id', 'id', 'job_schedule.job_specification')
        self.assertEqual(resolved, 'job_id')

        resolved = _command_type._build_prefix('properties_id', 'id', 'pool.start_task.properties')
        self.assertEqual(resolved, 'start_task_id')

        resolved = _command_type._build_prefix('start_task_id', 'id', 'pool.start_task.properties')
        self.assertEqual(resolved, 'pool_id')

        resolved = _command_type._build_prefix('pool_id', 'id', 'pool.start_task.properties')
        self.assertEqual(resolved, 'pool_id')

    def test_batch_find_param_type(self):
        model = TestObj()
        model.__doc__ = """
    :param name: The name of the environment variable.
    :type name: str
    :param value: The value of the environment variable.
"""
        self.assertEqual(_command_type.find_param_type(model, 'name'), 'str')

        model.__doc__ = """
        :param pool_get_options: Additional parameters for the operation
        :type pool_get_options: ~azure.batch.models.PoolGetOptions
        :param dict custom_headers: headers that will be added to the request
"""
        self.assertEqual(_command_type.find_param_type(model, 'pool_get_options'),
                         '~azure.batch.models.PoolGetOptions')

        model.__doc__ = '''
    :param node_fill_type: How tasks should be distributed across compute
     nodes. Possible values include: 'spread', 'pack', 'unmapped'
    :type node_fill_type: str or ~azure.batch.models.ComputeNodeFillType
    """
'''
        self.assertEqual(_command_type.find_param_type(model, 'node_fill_type'),
                         'str or ~azure.batch.models.ComputeNodeFillType')

        model.__doc__ = """
    :param name: The name of the environment variable.
    :type name:str
    :raises: BatchException
"""
        self.assertEqual(_command_type.find_param_type(model, 'name'), 'str')

    def test_batch_find_param_help(self):
        model = TestObj()
        model.__doc__ = """
        :param pool_id: The id of the pool to get.
        :type pool_id: str
        :param pool_get_options: Additional parameters for the operation
        :type pool_get_options: ~azure.batch.models.PoolGetOptions
"""
        self.assertEqual(_command_type.find_param_help(model, 'pool_id'),
                         'The id of the pool to get.')
        self.assertEqual(_command_type.find_param_help(model, 'pool_get_options'),
                         'Additional parameters for the operation')

        model.__doc__ = """
    :param node_fill_type: How tasks should be distributed across compute
     nodes. Possible values include: 'spread', 'pack'
    :type node_fill_type: str or ~azure.batch.models.ComputeNodeFillType
"""
        self.assertEqual(_command_type.find_param_help(model, 'node_fill_type'),
                         "How tasks should be distributed across compute nodes. " +
                         "Possible values include: 'spread', 'pack'")

    def test_batch_find_return_type(self):
        model = TestObj()
        model.__doc__ = """
    :param node_fill_type: How tasks should be distributed across compute
     nodes. Possible values include: 'spread', 'pack'
    :type node_fill_type: str or ~azure.batch.models.ComputeNodeFillType
"""
        self.assertIsNone(_command_type.find_return_type(model))

        model.__doc__ = """
        :type callback: Callable[Bytes, response=None]
        :param operation_config: :ref:`Operation configuration
         overrides<msrest:optionsforoperations>`.
        :rtype: Generator or
         ~msrest.pipeline.ClientRawResponse
"""
        self.assertEqual(_command_type.find_return_type(model), 'Generator')

    def test_batch_class_name(self):
        type_str = "~azure.batch.models.ComputeNodeFillType"
        self.assertEqual(_command_type.class_name(type_str),
                         "azure.batch.models.ComputeNodeFillType")

        type_str = "str or ~azure.batch.models.ComputeNodeFillType"
        self.assertEqual(_command_type.class_name(type_str),
                         "azure.batch.models.ComputeNodeFillType")

    def test_batch_operations_name(self):
        op_str = "PythonTestCase"
        self.assertEqual(_command_type.operations_name(op_str), "python_test_case")

        op_str = "PythonTestCaseOperations"
        self.assertEqual(_command_type.operations_name(op_str), "python_test_case")

        op_str = "Python"
        self.assertEqual(_command_type.operations_name(op_str), "python")

        op_str = "python"
        self.assertEqual(_command_type.operations_name(op_str), "python")

    def test_batch_full_name(self):
        arg_details = {'path': 'pool.start_task', 'root': 'id'}
        self.assertEqual(_command_type.full_name(arg_details), 'pool.start_task.id')

    def test_batch_group_title(self):
        path = "pool"
        self.assertEqual(_command_type.group_title(path), "Pool")

        path = "pool_patch_parameter"
        self.assertEqual(_command_type.group_title(path), "Pool")

        path = "pool_update_parameter"
        self.assertEqual(_command_type.group_title(path), "Pool")

        path = "pool_update_properties_parameter"
        self.assertEqual(_command_type.group_title(path), "Pool Update Properties")

        path = "pool.start_task"
        self.assertEqual(_command_type.group_title(path), "Pool: Start Task")

        path = "pool.start_task.constraints"
        self.assertEqual(_command_type.group_title(path), "Pool: Start Task: Constraints")

    def test_batch_arg_name(self):
        self.assertEqual(_command_type.arg_name("pool_id"), "--pool-id")
        self.assertEqual(_command_type.arg_name("id"), "--id")
        self.assertEqual(_command_type.arg_name("start_task_id"), "--start-task-id")

    def test_batch_format_options_name(self):
        op = "azure.batch.operations.pool_opterations#PoolOperations.get"
        self.assertEqual(_command_type.format_options_name(op), "pool_get_options")

        op = "azure.batch.operations.pool_opterations#PoolOperations.upgrade_os"
        self.assertEqual(_command_type.format_options_name(op), "pool_upgrade_os_options")

        op = "azure.batch.operations.pool_opterations#JobScheduleOperations.get"
        self.assertEqual(_command_type.format_options_name(op), "job_schedule_get_options")

    def test_batch_argument_tree(self):  # pylint: disable=too-many-statements
        tree = _command_type.BatchArgumentTree(None)
        self.assertEqual(list(tree), [])

        tree.set_request_param("pool", "azure.batch.models.PoolAddParameter")
        self.assertEqual(tree._request_param, {'name': 'pool', 'model': 'PoolAddParameter'})

        self.assertEqual(tree.dequeue_argument("id"), {})
        self.assertFalse(tree.existing("id"))

        tree.queue_argument('id', 'pool', 'id', {}, 'str', ['vm_size', 'id'])
        tree.queue_argument('vm_size', 'pool', 'vm_size', {}, 'str', ['vm_size', 'id'])
        tree.queue_argument('target_dedicated_nodes', 'pool', 'target_dedicated_nodes', {}, 'int',
                            ['vm_size', 'id'])
        tree.queue_argument('command_line', 'pool.start_task', 'command_line', {}, 'str',
                            ['command_line'])
        tree.queue_argument('run_elevated', 'pool.start_task', 'run_elevated', {}, 'bool',
                            ['command_line'])
        tree.queue_argument('node_agent_sku_id', 'pool.virtual_machine_configuration',
                            'node_agent_sku_id', {}, 'str',
                            ['node_agent_sku_id', 'image_reference.offer',
                             'image_reference.publisher'])
        tree.queue_argument('offer', 'pool.virtual_machine_configuration.image_reference',
                            'offer', {}, 'str', ['offer', 'publisher'])
        tree.queue_argument('publisher', 'pool.virtual_machine_configuration.image_reference',
                            'publisher', {}, 'str', ['offer', 'publisher'])
        tree.queue_argument('version', 'pool.virtual_machine_configuration.image_reference',
                            'version', {}, 'str', ['offer', 'publisher'])
        tree.queue_argument('subnet_id', 'pool.network_configuration', 'id', {}, 'str', ['id'])
        tree.queue_argument('os_family', 'pool.cloud_service_configuration', 'os_family', {},
                            'str', ['os_family'])
        tree.queue_argument('target_os_version', 'pool.cloud_service_configuration',
                            'target_os_version', {}, 'str', ['os_family'])

        self.assertEqual(len(list(tree)), 12)
        self.assertTrue(tree.existing('vm_size'))

        ns = TestObj()
        ns.id = None
        ns.vm_size = None
        ns.target_dedicated_nodes = 3
        ns.command_line = None
        ns.run_elevated = None
        ns.node_agent_sku_id = None
        ns.offer = None
        ns.publisher = None
        ns.version = None
        ns.subnet_id = None
        ns.os_family = None
        ns.target_os_version = None
        with self.assertRaises(ValueError):
            tree.parse(ns)
        ns.id = "test_pool"
        with self.assertRaises(ValueError):
            tree.parse(ns)
        ns.vm_size = "small"
        tree.parse(ns)
        ns.run_elevated = True
        with self.assertRaises(ValueError):
            tree.parse(ns)
        ns.command_line = "cmd"
        tree.parse(ns)
        ns.run_elevated = None
        tree.parse(ns)
        ns.offer = "offer"
        with self.assertRaises(ValueError):
            tree.parse(ns)
        ns.publisher = "publisher"
        with self.assertRaises(ValueError):
            tree.parse(ns)
        ns.node_agent_sku_id = "sku id"
        tree.parse(ns)

        with self.assertRaises(ValueError):
            tree.parse_mutually_exclusive(ns, False, ['pool.id', 'pool.vm_size'])
        ns.id = None
        tree.parse_mutually_exclusive(ns, False, ['pool.id', 'pool.vm_size'])
        ns.vm_size = None
        tree.parse_mutually_exclusive(ns, False, ['pool.id', 'pool.vm_size'])
        with self.assertRaises(ValueError):
            tree.parse_mutually_exclusive(ns, True, ['pool.id', 'pool.vm_size'])

        ns.id = None
        tree.parse_mutually_exclusive(ns, False, ['pool.id', 'pool.cloud_service_configuration'])
        with self.assertRaises(ValueError):
            tree.parse_mutually_exclusive(
                ns, True, ['pool.id', 'pool.cloud_service_configuration'])
        ns.id = "id"
        tree.parse_mutually_exclusive(
            ns, True, ['pool.id', 'pool.cloud_service_configuration'])
        ns.target_os_version = "4"
        with self.assertRaises(ValueError):
            tree.parse_mutually_exclusive(
                ns, True, ['pool.id', 'pool.cloud_service_configuration'])

        with self.assertRaises(ValueError):
            tree.parse_mutually_exclusive(
                ns, True, ['pool.virtual_machine_configuration',
                           'pool.cloud_service_configuration'])
        ns.target_os_version = None
        tree.parse_mutually_exclusive(
            ns, True, ['pool.virtual_machine_configuration',
                       'pool.cloud_service_configuration'])
        ns.publisher = None
        ns.offer = None
        ns.node_agent_sku_id = None
        tree.parse_mutually_exclusive(
            ns, False, ['pool.virtual_machine_configuration',
                        'pool.cloud_service_configuration'])
        with self.assertRaises(ValueError):
            tree.parse_mutually_exclusive(ns, True, ['pool.virtual_machine_configuration',
                                                     'pool.cloud_service_configuration'])

        siblings = tree._get_siblings("pool")
        self.assertEqual(sorted(siblings), ["id", "target_dedicated_nodes", "vm_size"])
        siblings = tree._get_siblings("pool.virtual_machine_configuration")
        self.assertEqual(sorted(siblings), ["node_agent_sku_id"])
        children = tree._get_children("pool.virtual_machine_configuration")
        self.assertEqual(sorted(children), ["node_agent_sku_id", "offer", "publisher", "version"])

        tree.dequeue_argument('node_agent_sku_id')
        self.assertEqual(len(list(tree)), 11)


class TestBatchLoader(unittest.TestCase):  # pylint: disable=protected-access

    def setUp(self):
        def get_client(*_):
            creds = SharedKeyCredentials('test1', 'ZmFrZV9hY29jdW50X2tleQ==')
            return BatchServiceClient(creds, 'https://test1.westus.batch.azure.com/')

        self.command_pool = _command_type.AzureBatchDataPlaneCommand(
            'azure.batch.operations.pool_operations#PoolOperations.add',
            operations.pool_operations.PoolOperations.add,
            client_factory=get_client)
        self.command_node = _command_type.AzureBatchDataPlaneCommand(
            'azure.batch.operations.compute_node_operations#ComputeNodeOperations.reboot',
            operations.compute_node_operations.ComputeNodeOperations.reboot,
            client_factory=get_client)
        self.command_job = _command_type.AzureBatchDataPlaneCommand(
            'azure.batch.operations.job_operations#JobOperations.add',
            operations.job_operations.JobOperations.add,
            client_factory=get_client)
        self.command_task = _command_type.AzureBatchDataPlaneCommand(
            'azure.batch.operations.task_operations#TaskOperations.add',
            operations.task_operations.TaskOperations.add,
            client_factory=get_client, flatten=1)
        self.command_file = _command_type.AzureBatchDataPlaneCommand(
            'azure.batch.operations.file_operations#FileOperations.get_from_task',
            operations.file_operations.FileOperations.get_from_task,
            client_factory=get_client)
        self.command_list = _command_type.AzureBatchDataPlaneCommand(
            'azure.batch.operations.job_operations#JobOperations.list',
            operations.job_operations.JobOperations.list,
            client_factory=get_client)
        self.command_delete = _command_type.AzureBatchDataPlaneCommand(
            'azure.batch.operations.pool_operations#PoolOperations.delete',
            operations.pool_operations.PoolOperations.delete,
            client_factory=get_client)
        self.command_conflicts = _command_type.AzureBatchDataPlaneCommand(
            'azure.batch.operations.job_schedule_operations#JobScheduleOperations.add',
            operations.job_schedule_operations.JobScheduleOperations.add,
            client_factory=get_client, flatten=4)
        return super(TestBatchLoader, self).setUp()

    def test_batch_build_parameters(self):
        kwargs = {
            'id': 'poolid',
            'vm_size': 'small',
            'os_family': '4',
            'run_elevated': True,
            'command_line': 'cmd',
            'wait_for_success': None
        }
        params = {
            'id': {'path': 'pool', 'root': 'id'},
            'vm_size': {'path': 'pool', 'root': 'vm_size'},
            'os_family': {'path': 'pool.cloud_service_configuration', 'root': 'os_family'},
            'run_elevated': {'path': 'pool.start_task', 'root': 'run_elevated'},
            'command_line': {'path': 'pool.start_task', 'root': 'command_line'},
            'wait_for_success': {'path': 'pool.start_task', 'root': 'wait_for_success'}
        }
        for arg, details in params.items():
            value = kwargs.pop(arg)
            if value is None:
                continue
            params = self.command_pool._build_parameters(
                details['path'],
                kwargs,
                details['root'],
                value)

        request = {'pool': {'id': 'poolid', 'vm_size': 'small',
                            'cloud_service_configuration': {'os_family': '4'},
                            'start_task': {'run_elevated': True, 'command_line': 'cmd'}}}
        self.assertEqual(kwargs, request)

    def test_batch_options(self):
        self.command_delete._load_options_model(
            operations.pool_operations.PoolOperations.delete)
        self.assertIsInstance(self.command_delete._options_model, models.PoolDeleteOptions)
        self.assertEqual(sorted(self.command_delete._options_attrs),
                         ['additional_properties',
                          'client_request_id',
                          'if_match',
                          'if_modified_since',
                          'if_none_match',
                          'if_unmodified_since',
                          'ocp_date',
                          'return_client_request_id',
                          'timeout'])
        kwargs = {
            'if_match': None,
            'if_modified_since': 'abc',
            'if_none_match': None,
            'if_unmodified_since': 'def',
            'client_request_id': 'ignored'
        }
        self.command_delete._build_options(kwargs)
        self.assertIsInstance(kwargs['pool_delete_options'], models.PoolDeleteOptions)
        self.assertEqual(kwargs['pool_delete_options'].if_modified_since, 'abc')
        self.assertEqual(kwargs['pool_delete_options'].if_unmodified_since, 'def')
        self.assertIsNone(kwargs['pool_delete_options'].client_request_id)
        self.assertEqual(kwargs['pool_delete_options'].timeout, 30)
        self.assertIsNone(kwargs['pool_delete_options'].ocp_date)
        options = list(self.command_delete._process_options())
        self.assertEqual(len(options), 4)

    def test_batch_should_flatten(self):
        self.assertFalse(self.command_task._should_flatten('task.depends_on'))
        self.assertTrue(self.command_task._should_flatten('task'))
        self.assertFalse(self.command_job._should_flatten(
            'job.job_manager_task.constraints.something'))
        self.assertTrue(self.command_job._should_flatten('job.job_manager_task.constraints'))

    def test_batch_get_model_attrs(self):
        self.command_job.parser = mock.Mock(_request_param={'name': 'job'})
        attrs = list(self.command_job._get_attrs(models.ResourceFile, 'task.resource_files'))
        self.assertEqual(len(attrs), 3)
        attrs = list(self.command_job._get_attrs(models.JobManagerTask, 'job.job_manager_task'))
        self.assertEqual(len(attrs), 6)
        attrs = list(self.command_job._get_attrs(models.JobAddParameter, 'job'))
        self.assertEqual(len(attrs), 7)

    def test_batch_load_arguments(self):
        # pylint: disable=too-many-statements
        handler = operations.pool_operations.PoolOperations.add
        args = list(self.command_pool._load_transformed_arguments(handler))
        self.assertEqual(len(args), 27)
        self.assertFalse('yes' in [a for a, _ in args])
        self.assertTrue('json_file' in [a for a, _ in args])
        self.assertFalse('destination' in [a for a, _ in args])
        self.assertTrue('application_package_references' in [a for a, _ in args])
        self.assertFalse('start_task_environment_settings' in [a for a, _ in args])
        self.assertTrue('certificate_references' in [a for a, _ in args])
        self.assertTrue('metadata' in [a for a, _ in args])
        handler = operations.job_operations.JobOperations.add
        args = list(self.command_job._load_transformed_arguments(handler))
        self.assertEqual(len(args), 16)
        self.assertFalse('yes' in [a for a, _ in args])
        self.assertTrue('json_file' in [a for a, _ in args])
        self.assertFalse('destination' in [a for a, _ in args])
        handler = operations.task_operations.TaskOperations.add
        args = list(self.command_task._load_transformed_arguments(handler))
        self.assertEqual(len(args), 11)
        self.assertFalse('yes' in [a for a, _ in args])
        self.assertTrue('json_file' in [a for a, _ in args])
        self.assertFalse('destination' in [a for a, _ in args])
        handler = operations.file_operations.FileOperations.get_from_task
        args = list(self.command_file._load_transformed_arguments(handler))
        self.assertEqual(len(args), 12)
        self.assertFalse('yes' in [a for a, _ in args])
        self.assertFalse('json_file' in [a for a, _ in args])
        self.assertTrue('destination' in [a for a, _ in args])
        handler = operations.job_operations.JobOperations.list
        args = list(self.command_list._load_transformed_arguments(handler))
        self.assertEqual(len(args), 7)
        names = [a for a, _ in args]
        self.assertEqual(set(names), set(['filter', 'select', 'expand', 'cmd', 'account_name', 'account_key', 'account_endpoint']))
        self.assertFalse('yes' in [a for a, _ in args])
        self.assertFalse('json_file' in [a for a, _ in args])
        self.assertFalse('destination' in [a for a, _ in args])
        handler = operations.pool_operations.PoolOperations.delete
        args = list(self.command_delete._load_transformed_arguments(handler))
        self.assertEqual(len(args), 10)
        self.assertTrue('yes' in [a for a, _ in args])
        self.assertFalse('json_file' in [a for a, _ in args])
        self.assertFalse('destination' in [a for a, _ in args])
        handler = operations.job_schedule_operations.JobScheduleOperations.add
        args = [a for a, _ in self.command_conflicts._load_transformed_arguments(handler)]
        self.assertEqual(len(args), 20)
        self.assertTrue('id' in args)
        self.assertTrue('job_manager_task_id' in args)
        self.assertFalse('job_manager_task_max_wall_clock_time' in args)
        self.assertTrue('job_max_wall_clock_time' in args)
        self.assertFalse('allow_low_priority_node' in args)
        self.assertFalse('yes' in args)
        self.assertTrue('json_file' in args)
        self.assertFalse('destination' in args)
        handler = operations.compute_node_operations.ComputeNodeOperations.reboot
        args = list(self.command_node._load_transformed_arguments(handler))
        self.assertEqual(len(args), 7)
        self.assertTrue('node_reboot_option' in [a for a, _ in args])
        option = [arg for (name, arg) in args if name == 'node_reboot_option'][0]
        self.assertIsNotNone(option.choices)
        self.assertFalse([a for a in option.choices if "'" in a])



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer, StorageAccountPreparer


class HDInsightClusterTests(ScenarioTest):
    location = 'eastus'

    @ResourceGroupPreparer(name_prefix='hdicli-', location=location, random_name_length=12)  # Uses 'rg' kwarg
    @StorageAccountPreparer(name_prefix='hdicli', location=location, parameter_name='storage_account')
    def test_hdinsight_cluster_min_args(self, storage_account_info):
        self._create_hdinsight_cluster(self._wasb_arguments(storage_account_info,
                                                            specify_key=False, specify_container=False))

    @ResourceGroupPreparer(name_prefix='hdicli-', location=location, random_name_length=12)  # Uses 'rg' kwarg
    @StorageAccountPreparer(name_prefix='hdicli', location=location, parameter_name='storage_account')
    def test_hdinsight_cluster_resize(self, storage_account_info):
        self._create_hdinsight_cluster(self._wasb_arguments(storage_account_info))

        resize_cluster_format = 'az hdinsight resize -n {name} -g {rg} --target-instance-count 2'
        self.cmd(resize_cluster_format)

        self.cmd('az hdinsight show -n {name} -g {rg}', checks=[
            self.check('properties.provisioningState', 'Succeeded'),
            self.check('properties.clusterState', 'Running'),
            self.check("properties.computeProfile.roles[?name=='workernode'].targetInstanceCount", [2])
        ])

    @ResourceGroupPreparer(name_prefix='hdicli-', location=location, random_name_length=12)  # Uses 'rg' kwarg
    @StorageAccountPreparer(name_prefix='hdicli', location=location, parameter_name='storage_account')
    def test_hdinsight_cluster_kafka(self, storage_account_info):
        self._create_hdinsight_cluster(
            HDInsightClusterTests._wasb_arguments(storage_account_info),
            HDInsightClusterTests._kafka_arguments()
        )

    @ResourceGroupPreparer(name_prefix='hdicli-', location=location, random_name_length=12)  # Uses 'rg' kwarg
    @StorageAccountPreparer(name_prefix='hdicli', location=location, parameter_name='storage_account')
    def test_hdinsight_cluster_kafka_with_optional_disk_args(self, storage_account_info):
        self._create_hdinsight_cluster(
            HDInsightClusterTests._wasb_arguments(storage_account_info),
            HDInsightClusterTests._kafka_arguments(),
            HDInsightClusterTests._optional_data_disk_arguments()
        )

    @ResourceGroupPreparer(name_prefix='hdicli-', location=location, random_name_length=12)  # Uses 'rg' kwarg
    @StorageAccountPreparer(name_prefix='hdicli', location=location, parameter_name='storage_account')
    def test_hdinsight_cluster_rserver(self, storage_account_info):
        self._create_hdinsight_cluster(
            HDInsightClusterTests._wasb_arguments(storage_account_info),
            HDInsightClusterTests._rserver_arguments()
        )

    @ResourceGroupPreparer(name_prefix='hdicli-', location=location, random_name_length=12)  # Uses 'rg' kwarg
    @StorageAccountPreparer(name_prefix='hdicli', location=location, parameter_name='storage_account')
    def test_hdinsight_cluster_with_component_version(self, storage_account_info):
        self._create_hdinsight_cluster(
            HDInsightClusterTests._wasb_arguments(storage_account_info),
            HDInsightClusterTests._component_version_arguments()
        )

    @ResourceGroupPreparer(name_prefix='hdicli-', location=location, random_name_length=12)  # Uses 'rg' kwarg
    @StorageAccountPreparer(name_prefix='hdicli', location=location, parameter_name='storage_account')
    def test_hdinsight_cluster_with_cluster_config(self, storage_account_info):
        self._create_hdinsight_cluster(
            HDInsightClusterTests._wasb_arguments(storage_account_info),
            HDInsightClusterTests._with_cluster_config()
        )

    @ResourceGroupPreparer(name_prefix='hdicli-', location=location, random_name_length=12)  # Uses 'rg' kwarg
    @StorageAccountPreparer(name_prefix='hdicli', location=location, parameter_name='storage_account')
    def test_hdinsight_cluster_with_ssh_creds(self, storage_account_info):
        self._create_hdinsight_cluster(
            HDInsightClusterTests._wasb_arguments(storage_account_info),
            HDInsightClusterTests._with_explicit_ssh_creds()
        )

    def _create_hdinsight_cluster(self, *additional_create_arguments):
        self.kwargs.update({
            'loc': self.location,
            'name': self.create_random_name(prefix='hdicli-', length=16),
            'http-password': 'Password1!'
        })

        create_cluster_format = 'az hdinsight create -n {name} -g {rg} -l {loc} -p {http-password} ' \
                                + ' '.join(additional_create_arguments)
        self.cmd(create_cluster_format, checks=[
            self.check('properties.provisioningState', 'Succeeded'),
            self.check('properties.clusterState', 'Running'),
            self.check("properties.computeProfile.roles[?name=='headnode']"
                       ".osProfile.linuxOperatingSystemProfile.username", ['sshuser'])
        ])

        self.cmd('az hdinsight show -n {name} -g {rg}', checks=[
            self.check('properties.provisioningState', 'Succeeded'),
            self.check('properties.clusterState', 'Running')
        ])

    @staticmethod
    def _wasb_arguments(storage_account_info, specify_key=True, specify_container=True):
        storage_account_name, storage_account_key = storage_account_info
        storage_account_key = storage_account_key.strip()

        key_args = ' --storage-account-key "{}"'.format(storage_account_key) if specify_key else ""
        container_args = ' --storage-default-container {}'.format('default') if specify_container else ""

        return '--storage-account {}.blob.core.windows.net{}{}'\
            .format(storage_account_name, key_args, container_args)

    @staticmethod
    def _kafka_arguments():
        return '-t {} --workernode-data-disks-per-node {}'.format('kafka', '4')

    @staticmethod
    def _optional_data_disk_arguments():
        return '--workernode-data-disk-storage-account-type {} --workernode-data-disk-size {}'\
               .format('Standard_LRS', '1023')

    @staticmethod
    def _rserver_arguments():
        return '-t {} --edgenode-size {} -v 3.6'.format('rserver', 'large')

    @staticmethod
    def _component_version_arguments():
        return '-t {} --component-version {}'.format('spark', 'spark=2.2')

    @staticmethod
    def _with_cluster_config():
        return '--cluster-configurations {}'.format(r'{{\"gateway\":{{\"restAuthCredential.username\":\"admin\"}}}}')

    @staticmethod
    def _with_explicit_ssh_creds():
        return '--ssh-user {} --ssh-password {}'.format('sshuser', 'Password1!')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer
import uuid


class AuthSettingTests(ScenarioTest):

    @ResourceGroupPreparer(random_name_length=20)
    def test_auth_setting(self, resource_group):
        self.kwargs.update({
            'botname': self.create_random_name(prefix='cli', length=10),
            'endpoint': 'https://www.google.com/api/messages',
            'app_id': str(uuid.uuid4()),
            'setting_name': self.create_random_name(prefix='auth', length=10),
            'clientid': 'clientid',
            'secret': 'secret'
        })

        self.cmd('az bot create -k registration -g {rg} -n {botname} -e {endpoint} --appid {app_id}', checks=[
            self.check('name', '{botname}'),
            self.check('resourceGroup', '{rg}'),
            self.check('location', 'global')
        ])

        self.cmd('az bot authsetting create -g {rg} -n {botname} --client-id {clientid} --client-secret {secret} --provider-scope-string "scope1 scope2" --service google -c myconnname', checks=[
            self.check('properties.clientId', '{clientid}')
        ])

        self.cmd('az bot authsetting show -c myconnname -g {rg} -n {botname}', checks=[
            self.check('properties.clientId', '{clientid}')
        ])

        self.cmd('az bot authsetting list -n {botname} -g {rg}', checks=[
            self.check('length(@)', 1),
            self.check('[0].properties.name', 'myconnname')
        ])

        self.cmd('az bot authsetting delete -g {rg} -n {botname} -c myconnname')

    def test_service_providers(self):
        self.kwargs.update({
            'service_provider': 'google'
        })

        self.cmd('az bot authsetting list-providers', checks=[
            self.greater_than('length(@)', 0)
        ])

        self.cmd('az bot authsetting list-providers --provider-name {service_provider}', checks=[
            self.check('length(@)', 1),
            self.check('properties.serviceProviderName', '{service_provider}')
        ])



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import unittest
import mock

from azure.cli.command_modules.role.custom import _resolve_role_id

# pylint: disable=line-too-long


class TestRoleCustomCommands(unittest.TestCase):

    def test_resolve_role_id(self, ):
        mock_client = mock.Mock()
        mock_client.config.subscription_id = '123'
        test_role_id = 'b24988ac-6180-42a0-ab88-20f738123456'

        # action(using a logical name)
        result = _resolve_role_id(test_role_id, 'foobar', mock_client)

        # assert
        self.assertEqual('/subscriptions/123/providers/Microsoft.Authorization/roleDefinitions/{}'.format(test_role_id), result)

        # action (using a full id)
        test_full_id = '/subscriptions/0b1f6471-1bf0-4dda-aec3-cb9272123456/providers/microsoft.authorization/roleDefinitions/5370bbf4-6b73-4417-969b-8f2e6e123456'
        self.assertEqual(test_full_id, _resolve_role_id(test_full_id, 'foobar', mock_client))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer, StorageAccountPreparer


class TestLogProfileScenarios(ScenarioTest):
    @ResourceGroupPreparer(location='southcentralus')
    @StorageAccountPreparer(location='southcentralus')
    def test_monitor_create_log_profile(self, resource_group, storage_account):
        self.kwargs.update({
            'name': self.create_random_name('clitest', 20)
        })
        self.kwargs['storage'] = self.cmd('storage account show -n {sa} -g {rg} --query id -otsv').output.strip()
        self.cmd("monitor log-profiles create --categories 'Write' --enabled false --days 1095 --location southcentralus --locations westus southcentralus --name {name} --storage-account-id {storage}", checks=[
            self.check('storageAccountId', '{storage}'),
            self.check('serviceBusRuleId', None),
            self.check('retentionPolicy.enabled', 'False')
        ])
        self.cmd("monitor log-profiles delete -n {name}")



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.util import CLIError
from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer, StorageAccountPreparer
from azure.cli.command_modules.ams._test_utils import _get_test_data_file


class AmsAccountFilterTests(ScenarioTest):
    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_for_create')
    def test_ams_account_filter_create_and_show(self, storage_account_for_create):
        amsname = self.create_random_name(prefix='ams', length=12)
        filter_name = self.create_random_name(prefix='filter', length=12)

        self.kwargs.update({
            'amsname': amsname,
            'storageAccount': storage_account_for_create,
            'location': 'westus2',
            'filterName': filter_name,
            'bitrate': 420,
            'endTimestamp': 100000000,
            'liveBackoffDuration': 60,
            'presentationWindowDuration': 1200000000,
            'startTimestamp': 40000000,
            'timescale': 10000000,
            'tracks': '@' + _get_test_data_file('filterTracks.json')
        })

        self.cmd('az ams account create -n {amsname} -g {rg} --storage-account {storageAccount} -l {location}')

        self.cmd('az ams account-filter create -a {amsname} -g {rg} -n {filterName} --bitrate {bitrate} --end-timestamp {endTimestamp} --live-backoff-duration {liveBackoffDuration} --presentation-window-duration {presentationWindowDuration} --start-timestamp {startTimestamp} --timescale {timescale} --tracks "{tracks}"', checks=[
            self.check('firstQuality.bitrate', '{bitrate}'),
            self.check('name', '{filterName}'),
            self.check('presentationTimeRange.endTimestamp', '{endTimestamp}'),
            self.check('presentationTimeRange.liveBackoffDuration', '{liveBackoffDuration}'),
            self.check('presentationTimeRange.presentationWindowDuration', '{presentationWindowDuration}'),
            self.check('presentationTimeRange.startTimestamp', '{startTimestamp}'),
            self.check('presentationTimeRange.timescale', '{timescale}'),
            self.check('tracks[0].trackSelections[0].operation', 'Equal'),
            self.check('tracks[0].trackSelections[0].property', 'FourCC'),
            self.check('tracks[0].trackSelections[0].value', 'AVC1'),
            self.check('tracks[1].trackSelections[0].operation', 'NotEqual'),
            self.check('tracks[1].trackSelections[0].property', 'Unknown'),
            self.check('tracks[1].trackSelections[0].value', 'EC-3'),
            self.check('tracks[1].trackSelections[1].operation', 'Equal'),
            self.check('tracks[1].trackSelections[1].property', 'FourCC'),
            self.check('tracks[1].trackSelections[1].value', 'MP4A')
        ])

        self.cmd('az ams account-filter show -a {amsname} -g {rg} -n {filterName}', checks=[
            self.check('firstQuality.bitrate', '{bitrate}'),
            self.check('name', '{filterName}'),
            self.check('presentationTimeRange.endTimestamp', '{endTimestamp}'),
            self.check('presentationTimeRange.liveBackoffDuration', '{liveBackoffDuration}'),
            self.check('presentationTimeRange.presentationWindowDuration', '{presentationWindowDuration}'),
            self.check('presentationTimeRange.startTimestamp', '{startTimestamp}'),
            self.check('presentationTimeRange.timescale', '{timescale}'),
            self.check('tracks[0].trackSelections[0].operation', 'Equal'),
            self.check('tracks[0].trackSelections[0].property', 'FourCC'),
            self.check('tracks[0].trackSelections[0].value', 'AVC1'),
            self.check('tracks[1].trackSelections[0].operation', 'NotEqual'),
            self.check('tracks[1].trackSelections[0].property', 'Unknown'),
            self.check('tracks[1].trackSelections[0].value', 'EC-3'),
            self.check('tracks[1].trackSelections[1].operation', 'Equal'),
            self.check('tracks[1].trackSelections[1].property', 'FourCC'),
            self.check('tracks[1].trackSelections[1].value', 'MP4A')
        ])

    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_for_list_and_delete')
    def test_ams_account_filter_list_and_delete(self, storage_account_for_list_and_delete):
        amsname = self.create_random_name(prefix='ams', length=12)
        filter_name_1 = self.create_random_name(prefix='filter', length=12)
        filter_name_2 = self.create_random_name(prefix='filter', length=13)

        self.kwargs.update({
            'amsname': amsname,
            'storageAccount': storage_account_for_list_and_delete,
            'location': 'westus2',
            'filterName1': filter_name_1,
            'filterName2': filter_name_2,
            'bitrate1': 420,
            'bitrate2': 1000,
            'endTimestamp': 100000000,
            'liveBackoffDuration': 60,
            'presentationWindowDuration': 1200000000,
            'startTimestamp': 40000000,
            'timescale': 10000000,
            'tracks': '@' + _get_test_data_file('filterTracks.json')
        })

        self.cmd('az ams account create -n {amsname} -g {rg} --storage-account {storageAccount} -l {location}')

        self.cmd('az ams account-filter list -a {amsname} -g {rg}', checks=[
            self.check('length(@)', 0)
        ])

        self.cmd('az ams account-filter create -a {amsname} -g {rg} -n {filterName1} --bitrate {bitrate1} --end-timestamp {endTimestamp} --live-backoff-duration {liveBackoffDuration} --presentation-window-duration {presentationWindowDuration} --start-timestamp {startTimestamp} --timescale {timescale} --tracks "{tracks}"')

        self.cmd('az ams account-filter list -a {amsname} -g {rg}', checks=[
            self.check('length(@)', 1)
        ])

        self.cmd('az ams account-filter create -a {amsname} -g {rg} -n {filterName2} --bitrate {bitrate2} --end-timestamp {endTimestamp} --live-backoff-duration {liveBackoffDuration} --presentation-window-duration {presentationWindowDuration} --start-timestamp {startTimestamp} --timescale {timescale} --tracks "{tracks}"')

        self.cmd('az ams account-filter list -a {amsname} -g {rg}', checks=[
            self.check('length(@)', 2)
        ])

        self.cmd('az ams account-filter delete -a {amsname} -g {rg} -n {filterName2}')

        self.cmd('az ams account-filter list -a {amsname} -g {rg}', checks=[
            self.check('length(@)', 1)
        ])

    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_for_update')
    def test_ams_account_filter_update(self, storage_account_for_update):
        amsname = self.create_random_name(prefix='ams', length=12)
        filter_name = self.create_random_name(prefix='filter', length=12)

        self.kwargs.update({
            'amsname': amsname,
            'storageAccount': storage_account_for_update,
            'location': 'westus2',
            'filterName': filter_name,
            'bitrate': 420,
            'endTimestamp': 100000000,
            'liveBackoffDuration': 60,
            'presentationWindowDuration': 1200000000,
            'startTimestamp': 40000000,
            'timescale': 10000000,
            'tracks': '@' + _get_test_data_file('filterTracks.json'),
            'newBitrate': 1000,
            'newStartTimestamp': 40000001,
            'newEndTimestamp': 100000001,
            'newTrackOperation': 'Equal',
            'newTrackProperty': 'FourCC',
            'newTrackValue': 'EC-3'
        })
        self.cmd('az ams account create -n {amsname} -g {rg} --storage-account {storageAccount} -l {location}')

        self.cmd('az ams account-filter create -a {amsname} -g {rg} -n {filterName} --bitrate {bitrate} --end-timestamp {endTimestamp} --live-backoff-duration {liveBackoffDuration} --presentation-window-duration {presentationWindowDuration} --start-timestamp {startTimestamp} --timescale {timescale} --tracks "{tracks}"')

        self.cmd('az ams account-filter update -a {amsname} -g {rg} -n {filterName} --bitrate {newBitrate} --start-timestamp {newStartTimestamp} --end-timestamp {newEndTimestamp} --set tracks[1].trackSelections[0].operation={newTrackOperation} tracks[1].trackSelections[0].property={newTrackProperty} tracks[1].trackSelections[0].value={newTrackValue}', checks=[
            self.check('firstQuality.bitrate', '{newBitrate}'),
            self.check('name', '{filterName}'),
            self.check('presentationTimeRange.endTimestamp', '{newEndTimestamp}'),
            self.check('presentationTimeRange.liveBackoffDuration', '{liveBackoffDuration}'),
            self.check('presentationTimeRange.presentationWindowDuration', '{presentationWindowDuration}'),
            self.check('presentationTimeRange.startTimestamp', '{newStartTimestamp}'),
            self.check('presentationTimeRange.timescale', '{timescale}'),
            self.check('tracks[0].trackSelections[0].operation', 'Equal'),
            self.check('tracks[0].trackSelections[0].property', 'FourCC'),
            self.check('tracks[0].trackSelections[0].value', 'AVC1'),
            self.check('tracks[1].trackSelections[0].operation', '{newTrackOperation}'),
            self.check('tracks[1].trackSelections[0].property', '{newTrackProperty}'),
            self.check('tracks[1].trackSelections[0].value', '{newTrackValue}'),
            self.check('tracks[1].trackSelections[1].operation', 'Equal'),
            self.check('tracks[1].trackSelections[1].property', 'FourCC'),
            self.check('tracks[1].trackSelections[1].value', 'MP4A')
        ])



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import tempfile
import shutil

from azure.cli.testsdk.preparers import AbstractPreparer


class StorageScenarioMixin(object):
    profile = None

    def get_current_profile(self):
        if not self.profile:
            self.profile = self.cmd('cloud show --query profile -otsv').output
        return self.profile

    def get_account_key(self, group, name):
        if self.get_current_profile() == '2017-03-09-profile':
            template = 'storage account keys list -n {} -g {} --query "key1" -otsv'
        else:
            template = 'storage account keys list -n {} -g {} --query "[0].value" -otsv'

        return self.cmd(template.format(name, group)).output

    def get_account_info(self, group, name):
        """Returns the storage account name and key in a tuple"""
        return name, self.get_account_key(group, name)

    def storage_cmd(self, cmd, account_info, *args):
        cmd = cmd.format(*args)
        cmd = '{} --account-name {} --account-key {}'.format(cmd, *account_info)
        return self.cmd(cmd)

    def storage_cmd_negative(self, cmd, account_info, *args):
        cmd = cmd.format(*args)
        cmd = '{} --account-name {} --account-key {}'.format(cmd, *account_info)
        return self.cmd(cmd, expect_failure=True)

    def create_container(self, account_info, prefix='cont', length=24):
        container_name = self.create_random_name(prefix=prefix, length=length)
        self.storage_cmd('storage container create -n {}', account_info, container_name)
        return container_name

    def create_share(self, account_info, prefix='share', length=24):
        share_name = self.create_random_name(prefix=prefix, length=length)
        self.storage_cmd('storage share create -n {}', account_info, share_name)
        return share_name


class StorageTestFilesPreparer(AbstractPreparer):
    def __init__(self, parameter_name='test_dir'):
        super(StorageTestFilesPreparer, self).__init__(name_prefix='test', name_len=24)
        self.parameter_name = parameter_name

    def create_resource(self, name, **kwargs):
        temp_dir = os.path.join(tempfile.gettempdir(), self.random_name)
        if not os.path.exists(temp_dir):
            os.mkdir(temp_dir)

        with open(os.path.join(temp_dir, 'readme'), 'w') as f:
            f.write('This directory contains test files generated by Azure CLI storage command '
                    'module tests.')

        for folder_name in ['apple', 'butter', 'butter/charlie', 'duff/edward']:
            for file_index in range(10):
                file_path = os.path.join(temp_dir, folder_name, 'file_%s' % file_index)
                if not os.path.exists(os.path.dirname(file_path)):
                    os.makedirs(os.path.dirname(file_path))

                with open(file_path, 'w') as f:
                    f.write('Azure CLI storage command module test sample file. origin:'
                            ' %s' % file_path)

        setattr(self, '_temp_dir', temp_dir)
        return {self.parameter_name: temp_dir}

    def remove_resource(self, name, **kwargs):
        temp_dir = self.get_temp_dir()
        if temp_dir:
            shutil.rmtree(temp_dir, ignore_errors=True)

    def get_temp_dir(self):
        return getattr(self, '_temp_dir', None)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest
import os.path
import mock
from six import StringIO

from knack.util import CLIError
from azure.cli.command_modules.resource._validators import (
    _validate_deployment_name,
    validate_lock_parameters,
)


class NamespaceObject:
    pass


class TestResourceValidators(unittest.TestCase):
    def setUp(self):
        self.io = StringIO()

    def tearDown(self):
        self.io.close()

    def test_validate_lock_params(self):
        valid = [
            {
                'test': 'just name',
                'name': 'foo'
            },
            {
                'test': 'name and group',
                'name': 'foo',
                'resource_group': 'foo',
            },
            {
                'test': 'name, group, type [compact]',
                'name': 'foo',
                'resource_group': 'bar',
                'resource_name': 'baz',
                'resource_type': 'Microsoft.Compute/VirtualMachines'
            },
            {
                'test': 'name, group, type, namespace',
                'name': 'foo',
                'resource_group': 'bar',
                'resource_name': 'baz',
                'resource_type': 'VirtualMachines',
                'resource_provider_namespace': 'Microsoft.Compute',
            },
            {
                'test': 'name, group, type, namespace, parent',
                'name': 'foo',
                'resource_group': 'bar',
                'resource_name': 'baz',
                'resource_type': 'VirtualMachines',
                'resource_provider_namespace': 'Microsoft.Compute',
                'parent_resource_path': 'Foo.Bar/baz',
            }
        ]
        for valid_namespace in valid:
            namespace_obj = NamespaceObject()
            for key in valid_namespace:
                setattr(namespace_obj, key, valid_namespace[key])
            try:
                # If unexpected invalid, this throws, so no need for asserts
                validate_lock_parameters(namespace_obj)
            except CLIError as ex:
                self.fail('Test {} failed. {}'.format(valid_namespace['test'], ex))

    def test_validate_lock_params_invalid(self):
        invalid = [
            {
                'test': 'just name and type',
                'name': 'foo',
                'resource_type': 'baz'
            },
            {
                'test': 'name and group and type',
                'name': 'foo',
                'resource_group': 'foo',
                'resource_type': 'bar',
            },
            {
                'test': 'name, group, no type',
                'name': 'foo',
                'resource_group': 'bar',
                'resource_name': 'baz',
            },
            {
                'test': 'name, group, type, namespace',
                'name': 'foo',
                'resource_group': 'bar',
                'resource_name': 'baz',
                'resource_provider_namespace': 'Microsoft.Compute',
            },
            {
                'test': 'name, group, type, namespace, parent',
                'name': 'foo',
                'resource_group': 'bar',
                'resource_type': 'VirtualMachines',
                'resource_provider_namespace': 'Microsoft.Compute',
                'parent_resource_path': 'Foo.Bar/baz',
            }
        ]
        for invalid_namespace in invalid:
            with self.assertRaises(CLIError):
                namespace_obj = NamespaceObject()
                for key in invalid_namespace:
                    setattr(namespace_obj, key, invalid_namespace[key])
                validate_lock_parameters(namespace_obj)

    def test_generate_deployment_name_from_file(self):
        # verify auto-gen from uri
        namespace = mock.MagicMock()
        namespace.template_uri = 'https://templates/template123.json?foo=bar'
        namespace.template_file = None
        namespace.deployment_name = None
        _validate_deployment_name(namespace)
        self.assertEqual('template123', namespace.deployment_name)

        namespace = mock.MagicMock()
        namespace.template_file = __file__
        namespace.template_uri = None
        namespace.deployment_name = None
        _validate_deployment_name(namespace)

        file_base_name = os.path.basename(__file__)
        file_base_name = file_base_name[:str.find(file_base_name, '.')]
        self.assertEqual(file_base_name, namespace.deployment_name)

        # verify use default if get a file content
        namespace = mock.MagicMock()
        namespace.template_file = '{"foo":"bar"}'
        namespace.template_uri = None
        namespace.deployment_name = None
        _validate_deployment_name(namespace)
        self.assertEqual('deployment1', namespace.deployment_name)


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer, StorageAccountPreparer


class TestMonitorMetrics(ScenarioTest):
    @ResourceGroupPreparer(location='southcentralus')
    @StorageAccountPreparer()
    def test_monitor_metrics_scenario(self, resource_group, storage_account):
        self.kwargs.update({})
        self.kwargs['sa_id'] = self.cmd('az storage account show -n {sa} -g {rg}').get_output_in_json()['id']
        self.cmd('az monitor metrics list-definitions --resource {sa_id} --namespace Microsoft.Storage/storageAccounts')
        self.cmd('az monitor metrics list --resource {sa_id} --namespace Microsoft.Storage/storageAccounts --metrics Ingress Egress --start-time 2018-01-01T00:00:00Z --end-time 2999-01-01T00:00:00Z',
                 checks=self.check('length(@.value)', 2))
        self.cmd('az monitor metrics list --resource {sa_id} --metrics Ingress Egress --start-time 2018-01-01 00:00:00 +00:00 --offset 5000d',
                 checks=self.check('length(@.value)', 2))
        self.cmd('az monitor metrics list --resource {sa_id} --metrics Ingress Egress --end-time 2025-01-01 00:00:00 +00:00 --offset 5000d',
                 checks=self.check('length(@.value)', 2))
